#!/usr/bin/python

"""
File: jbm.py

Description: A graphical utility for connecting to Wireless Broadband
			connections

License:
	jbm.py
	Copyright 2009 Jon Estey

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License version 2 as
	published by the Free Software Foundation

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	MA 02110-1301, USA.


Credits:

	Single Instance..
	zgoda @ http://stackoverflow.com/questions/380870/python-single-instance-of-program

"""

# **************** Import ****************
import wx
import os
import sys
import fcntl
from time import sleep
import Queue

# Import other project modules
from jbm_usage import *
from jbm_queue import *
from jbm_wvdial import *
from jbm_serial import *
from jbm_config import *

# **************** Globals ****************
main_path = os.path.expanduser("~/.jbm/")
usage_log_path = os.path.join(main_path,"usage")
pid_path = os.path.join(main_path,"pid")
config_path = os.path.join(main_path,"config")
wvdial_path = os.path.join(main_path, "wvdialrc")

modem_tty = None
dialer = None
message_queue = None

# **************** Classes ****************
class JBMSMSDialog(wx.Frame):
	"""Window for read/sending SMS messages"""
	
	def __init__(self, parent, id, read=False, SMS=None, reply_number=""):
		"""Initializes the dialog
		
		Parameters:
			parent, window parent
			
			id, window id
			
			read, If True the window is a reader if False the window
				is a sender
				
			SMS, If read is True SMS should contain the message that
				is to be read, other wise its ignored
				
			reply_number, If read is False number should contain the number
				number the message is directed to (for replys)
		"""
		
		# Make a title and set of default properties for controls
		if read:
			title = "SMS Message from <{0}>".format(SMS.number)
			number = SMS.number
			message = SMS.message
			attributes = wx.TE_READONLY
		else:
			title = "New SMS Message"
			number = reply_number
			message = ""
			attributes = 0
		
		# Initialize the frame
		wx.Frame.__init__(self, parent, id, title, size=(400,200))#, style=wx.DEFAULT_DIALOG_STYLE)

		# Create the main message displays and labels
		self.number_label = wx.StaticText(self, wx.ID_ANY, "Number:")		
		self.number_text = wx.TextCtrl(self, wx.ID_ANY, number, style=attributes)
		
		self.message_label = wx.StaticText(self, wx.ID_ANY, "Message:")
		self.message_text = wx.TextCtrl(self, wx.ID_ANY, message,
				style=attributes|wx.TE_MULTILINE)
		
		# Create a sizer for the window and add display elements to it
		self.sizer = wx.BoxSizer(wx.VERTICAL)
		
		self.sizer.Add(self.number_label, 0,
				wx.EXPAND|wx.TOP|wx.RIGHT|wx.LEFT, 5)
		self.sizer.Add(self.number_text, 0,
				wx.EXPAND|wx.RIGHT|wx.LEFT, 5)
		
		self.sizer.Add(self.message_label, 0,
				wx.EXPAND|wx.TOP|wx.RIGHT|wx.LEFT, 5)
		self.sizer.Add(self.message_text, 1,
				wx.EXPAND|wx.RIGHT|wx.LEFT, 5)
		
		# Create a sizer for the buttons
		self.button_sizer = wx.BoxSizer(wx.HORIZONTAL)
		
		# Add a spacer to the button sizer
		self.button_sizer.AddStretchSpacer()
		
		# Create buttons and add to the sizer depending on the mode
		if read:
			# When reading, close and reply buttons are required
			self.ID_REPLY = wx.NewId()
			self.reply_button = wx.Button(self, self.ID_REPLY, "Reply")
			
			self.ID_CLOSE = wx.NewId()
			self.close_button = wx.Button(self, self.ID_CLOSE, "Close")
			
			# Add to the button sizer
			self.button_sizer.Add(self.reply_button, 0, wx.EXPAND)
			self.button_sizer.Add(self.close_button, 0, wx.EXPAND)
		
			# Bind events
			wx.EVT_BUTTON(self, self.ID_REPLY, self.reply_click)
			wx.EVT_BUTTON(self, self.ID_CLOSE, self.close_click)
		else:
			# When sending, send and cancel buttons are required
			self.ID_SEND = wx.NewId()
			self.send_button = wx.Button(self, self.ID_SEND, "Send")
			
			self.ID_CANCEL = wx.NewId()
			self.cancel_button = wx.Button(self, self.ID_CANCEL, "Cancel")
			
			# Add to the button sizer
			self.button_sizer.Add(self.send_button, 0, wx.EXPAND)
			self.button_sizer.Add(self.cancel_button, 0,
					wx.EXPAND|wx.LEFT, 5)
			
			# Bind events
			wx.EVT_BUTTON(self, self.ID_SEND, self.send_click)
			wx.EVT_BUTTON(self, self.ID_CANCEL, self.close_click)
			
		# Add the button sizer to the main sizer
		self.sizer.Add(self.button_sizer, 0, wx.EXPAND|wx.ALL, 5)
		
		# Set the sizer for the window
		self.SetSizer(self.sizer)
		
		# Show the window
		self.Show(True)
		
	def close_click(self, event):
		"""Event for when the close or cancel buttons are clicked"""
		
		# Destroy the window
		self.Destroy()
	
	def reply_click(self, event):
		"""Event for when the close or cancel buttons are clicked"""
		
		# Create a new SMS dialog for reply
		JBMSMSDialog(self, wx.ID_ANY, reply_number=self.number_text.GetValue())
		
	def send_click(self, event):
		"""Event for when the send button is clicked"""
		
		if not modem_tty == None:
			# Send the message
			modem_tty.send(SC_SMS_Send(self.number_text.GetValue(),
					self.message_text.GetValue()))
			
		# Destroy the window
		self.Destroy()
		
class JBMMainWindow(wx.Frame):
	"""The primary display window for the application"""
	
	class ConnectionPanel(wx.Panel):
		def __init__(self, parent, id):
			"""Initiates the connection panel
			
			Parameters:
				parent, Parent window
				id, WX ID
			"""
			
			# Initiate the panel object
			wx.Panel.__init__(self, parent, id)

			# Create a sizer for the panel
			self.sizer = wx.BoxSizer(wx.VERTICAL)
			
			# Make the (dis)connect button
			self.ID_CONNECT = wx.NewId()
			self.connect_button = wx.Button(self, self.ID_CONNECT, "Connect",
				size=(300, 50))
			
			# Setup the buttons event
			wx.EVT_BUTTON(self, self.ID_CONNECT, self.connect_click)
			
			# Add the button to the sizer
			self.sizer.Add(self.connect_button, 0, wx.EXPAND|wx.ALL, 15)
			
			# Create information displays
			# Primary DNS, Secondary DNS, IP, Remote IP
			# Set the size of all labels to that of dns2 (longest one)
			self.dns2_title = wx.StaticText(self, wx.ID_ANY, "Secondary DNS:")
			self.dns2_disp = wx.TextCtrl(self, wx.ID_ANY, style=wx.TE_READONLY)
			(width, height) = self.dns2_title.GetBestSizeTuple()
			
			self.dns1_title = wx.StaticText(self, wx.ID_ANY, "Primary DNS:", size=(width, height))
			self.dns1_disp = wx.TextCtrl(self, wx.ID_ANY, style=wx.TE_READONLY)
			self.sizer.Add(self.make_hsizer(self.dns1_title, self.dns1_disp), 0, wx.EXPAND|wx.LEFT|wx.TOP|wx.RIGHT, 6)
			
			self.sizer.Add(self.make_hsizer(self.dns2_title, self.dns2_disp), 0, wx.EXPAND|wx.LEFT|wx.TOP|wx.RIGHT, 6)
			
			self.ip_title = wx.StaticText(self, wx.ID_ANY, "IP Address:",  size=(width, height))
			self.ip_disp = wx.TextCtrl(self, wx.ID_ANY, style=wx.TE_READONLY)
			self.sizer.Add(self.make_hsizer(self.ip_title, self.ip_disp), 0, wx.EXPAND|wx.LEFT|wx.TOP|wx.RIGHT, 6)
			
			self.remip_title = wx.StaticText(self, wx.ID_ANY, "Remote IP:", size=(width, height))
			self.remip_disp = wx.TextCtrl(self, wx.ID_ANY, style=wx.TE_READONLY)
			self.sizer.Add(self.make_hsizer(self.remip_title, self.remip_disp), 0, wx.EXPAND|wx.LEFT|wx.TOP|wx.RIGHT, 6)
			
			# Set the sizer for the panel
			self.SetSizer(self.sizer)
			
		def make_hsizer(self, label, display):
			"""Creates a horizontal sizer to contain the label and display"""
			
			# Create the horizontal sizer
			hsizer = wx.BoxSizer(wx.HORIZONTAL)
			
			# Add the label
			hsizer.Add(label, 0, wx.EXPAND|wx.RIGHT, 5)
			
			# Add the display
			hsizer.Add(display, 1, wx.EXPAND)
			
			# Return the sizer
			return hsizer
			
		def connect_click(self, event):
			"""Event handler for the connect button being clicked"""
			global dialer
			
			# Check to see if the button should connect or disconnect
			if self.connect_button.GetLabel() == "Connect":
				# Check to see if there is a modem connected
				if modem_tty == None:
					# Report the error
					ErrorDia = wx.MessageDialog(None, 
							"No valid modem found, please connect a valid modem and try again.",
							"Error", wx.OK | wx.ICON_ERROR)
					ErrorDia.ShowModal()
					ErrorDia.Destroy()
					
					return
				
				# Switch the label to indicate status
				self.connect_button.SetLabel("Connecting...")
				
				# Create a dialer if there isn't one already
				if dialer == None:
					dialer = WVDial(message_queue, wvdial_path, config)

				# Attempt to connect the modem
				dialer.dial()
			else:
				# Switch the label to indicate status
				self.connect_button.SetLabel("Disconnecting...")

				# Process pending events before waiting for disconnect
				#  to run
				wx.SafeYield()
				
				# Disconnect the modem
				dialer.disconnect()
							
	class UsagePanel(wx.Panel):
		"""The usage panel, contains a table filled with data of previous
		usage"""
		
		def __init__(self, parent, id):
			"""Initiates the usage log panel
			
			Parameters:
				parent, Parent window
				id, WX ID
			"""
			
			global usage_log_path
			
			# Initiate variables
			#self.usage_start = 0
			#self.usage_down = 0
			#self.usage_up = 0
			self.current_row = -1
			self.total_row = -1
			
			
			# Initiate the panel object
			wx.Panel.__init__(self, parent, id) 
			
			# Create a sizer for the panel
			self.sizer = wx.BoxSizer(wx.VERTICAL)
			
			# Create the main list view
			self.list = wx.ListCtrl(self, wx.ID_ANY,
				style=wx.LC_REPORT|wx.LC_SORT_ASCENDING|wx.LC_VRULES|wx.BORDER)
				
			self.sizer.Add(self.list, 1, wx.EXPAND|wx.ALL, 5)
			
			# Set the sizer
			self.SetSizer(self.sizer)
			
			# Insert List Columns
			self.list.InsertColumn(0, "Date")
			self.list.InsertColumn(1, "Duration")
			self.list.InsertColumn(2, "Up")
			self.list.InsertColumn(3, "Down")
			self.list.InsertColumn(4, "Total")
			
			# Load Usage
			self.usage = JBMUsage(usage_log_path, config)
			
			# Update the usage
			self.update_list(True)
		
		def update_list(self, update_all=False):
			"""Updates the usage list
			
			Parameters:
				update_all, if set performs a complete update
			"""

			# Get the current and total values
			cur = self.usage.get_current()
			tot = self.usage.get_total()
			
			# For a complete update delete all rows and start from scratch
			if update_all or self.total_row == -1:
				
				# Clear the list
				self.list.DeleteAllItems()
				
				# Add historic data
				for rec in self.usage:
					self.list.Append(rec)
			 
				# Add the current row if there is current data
				if not cur == None:
					self.current_row = self.list.Append(cur)
				else:
					self.current_row = -1
					
				# Add the totals row	
				self.total_row = self.list.Append(tot)
				
			else:
				# Only update the current row if it exists
				if self.current_row > -1:						
					# Update the current and total rows
					for i in range(1,5):
						self.list.SetStringItem(self.current_row,i,cur[i])
						self.list.SetStringItem(self.total_row,i,tot[i])
				else:
					# Update the total row
					for i in range(1,5):
						self.list.SetStringItem(self.total_row,i,tot[i])
			
			# Update column widths
			self.list.SetColumnWidth(0, wx.LIST_AUTOSIZE)
			self.list.SetColumnWidth(1, wx.LIST_AUTOSIZE)
			self.list.SetColumnWidth(2, wx.LIST_AUTOSIZE)
			self.list.SetColumnWidth(3, wx.LIST_AUTOSIZE)
			self.list.SetColumnWidth(4, wx.LIST_AUTOSIZE)
			
		def connected(self, interface):
			"""Function that should be called when the connection is
			opened
			
			Parameters:
				interface, the interface that was connected
			"""
			# Start the usage monitor
			self.usage.connect(interface)
			
			# Update the list
			self.update_list(True)
			
		def disconnected(self):
			"""Function that should be called when the connection is closed
			appends new usage data onto the usage log"""
			
			# Stop the usage monitor (saving data)
			self.usage.disconnect()
			
			# Update the list			
			self.update_list(True)
	
	class OperatorsPanel(wx.Panel):
		"""The operators panel, contains a table filled with various
		operators that have been found by the modem"""
		
		def __init__(self, parent, id):
			"""Initiates the usage log panel
			
			Parameters:
				parent, Parent window
				id, WX ID
			"""
			
			# Initiate the panel object
			wx.Panel.__init__(self, parent, id) 
			
			# Create a sizer for the panel
			self.sizer = wx.BoxSizer(wx.VERTICAL)
			
			# Create the main list view
			self.list = wx.ListCtrl(self, wx.ID_ANY,
				style=wx.LC_REPORT|wx.LC_SORT_ASCENDING|wx.LC_VRULES|wx.BORDER)
			
			self.sizer.Add(self.list, 1, wx.EXPAND|wx.ALL, 5)
			
			# Create a sizer to hold buttons
			self.button_sizer = wx.BoxSizer(wx.HORIZONTAL)
			
			# Create the scan and select buttons
			self.ID_SCAN = wx.NewId()
			self.scan_button = wx.Button(self, self.ID_SCAN, "Scan")
			
			self.ID_SELECT = wx.NewId()
			self.select_button = wx.Button(self, self.ID_SELECT, "Select")
			
			# Setup the button events
			wx.EVT_BUTTON(self, self.ID_SCAN, self.update_list)
			wx.EVT_BUTTON(self, self.ID_SELECT, self.select_click)
			
			# Add a spacer to the button sizer
			self.button_sizer.AddStretchSpacer()
			
			# Add the buttons to the button sizer
			self.button_sizer.Add(self.scan_button, 0, wx.EXPAND|wx.RIGHT, 5)
			self.button_sizer.Add(self.select_button, 0, wx.EXPAND, 0)
			
			# Add the button sizer to the main sizer
			self.sizer.Add(self.button_sizer, 0,
					wx.EXPAND|wx.LEFT|wx.RIGHT|wx.BOTTOM, 5)
			
			# Set the sizer
			self.SetSizer(self.sizer)
			
			# Insert List Columns
			self.list.InsertColumn(0, "Status",width=wx.LIST_AUTOSIZE)
			self.list.InsertColumn(1, "Operator",width=wx.LIST_AUTOSIZE)
			self.list.InsertColumn(2, "sname",width=0)
			self.list.InsertColumn(3, "nname", width=0)
			
			# Add information message
			#self.list.Append(["","Press scan to view available operators","",""])
			
			# Adjust column widths
			#self.list.SetColumnWidth(0, 0)
			#self.list.SetColumnWidth(1, wx.LIST_AUTOSIZE)
		
			
		def select_click(self, event):
			"""Select Button was clicked"""
			
			# Check that the modem tty is available
			global modem_tty
			if modem_tty == None:
				return
				
			# Get the numeric name of the selected operator
			selitem = self.list.GetFirstSelected()
			item = self.list.GetItem(selitem, 3)
			
			# Check for error and auto values
			if item.GetText() == "" or item.GetText() == "err" or item.GetText() == "auto":
				# Tell the modem to automatically select the operator
				modem_tty.send(SC_Op_Select())
				return
			
			# Tell the modem to select a specific operator
			modem_tty.send(SC_Op_Select(item.GetText()))
			
			
		def update_list(self, event=None):
			"""Updates the operator list
			
			Parameters:
			"""
						
			# Send the update command
			global modem_tty
			if not modem_tty == None:
				modem_tty.send(SC_Op_List(self.list_callback))
				
				# Disable the scan and select buttons
				self.scan_button.Enable(False)
				self.select_button.Enable(False)
				
		
		def list_callback(self, msg):
			"""Callback for the operator list"""

			# Enable the scan and select buttons
			self.scan_button.Enable(True)
			self.select_button.Enable(True)
			
			# Clear the current list
			self.list.DeleteAllItems()

			# Check for errors
			if msg.type == JBM_MSG_SER_ERROR or msg.type == JBM_MSG_SER_TIMEOUT:
				# Report the error
				self.list.Append(["","Error scanning for operators, please try again","err","err"])

				# Update column widths
				self.list.SetColumnWidth(0, 0)
				self.list.SetColumnWidth(1, wx.LIST_AUTOSIZE)	
			
			else:
				# Add auto selection
				self.list.Append(["","Automatic Selection","auto","auto"])
				
				# Add the new operators
				for operator in msg.data:
					self.list.Append(operator.list_item())
				
				# Update column widths
				self.list.SetColumnWidth(0, wx.LIST_AUTOSIZE)
				self.list.SetColumnWidth(1, wx.LIST_AUTOSIZE)	
					
			# Return false, no further processing required
			return False
			
		def active(self):
			"""Called when the panel is selected"""
			
			# Check to see if the there is an internet connection
			global dialer
			if not dialer == None:
				if dialer.is_connected():
					# There is a network connection, disable scanning
					#	and add a message to the list
					self.scan_button.Enable(False)
					self.select_button.Enable(False)
					
					self.list.DeleteAllItems()
					self.list.Append(["","Please disconnect if you would like to change operators","",""])
						
					self.list.SetColumnWidth(0, 0)
					self.list.SetColumnWidth(1, wx.LIST_AUTOSIZE)	
					return
					
			# No network connection set message accordingly			
			self.scan_button.Enable(True)
			self.select_button.Enable(True)
			
			self.list.DeleteAllItems()
			self.list.Append(["","Press scan to view available operators","",""])

			self.list.SetColumnWidth(0, 0)
			self.list.SetColumnWidth(1, wx.LIST_AUTOSIZE)	
			
	class ServicePanel(wx.Panel):
		def __init__(self, parent, id):
			"""Initiates the Service panel
			
			Parameters:
				parent, Parent window
				id, WX ID
			"""
			
			# Initiate the panel object
			wx.Panel.__init__(self, parent, id)	
			
			# Create the display panel
			self.disp_text = wx.TextCtrl(self, wx.ID_ANY,
				style=wx.TE_MULTILINE)
				
			# Create the input panel
			self.ID_INPUT = wx.NewId()
			self.input_text = wx.TextCtrl(self, self.ID_INPUT,
				style=wx.TE_PROCESS_ENTER)
			self.input_text.Enable(False)
			
			# Create the send button
			self.ID_SEND = wx.NewId()
			self.send_button = wx.Button(self, self.ID_SEND, "Send")
			self.send_button.Enable(False)
			
			# Create a sub sizer to hold input and send
			self.sub_sizer = wx.BoxSizer(wx.HORIZONTAL)
			
			# Add Input and Send Button to the sub sizer
			self.sub_sizer.Add(self.input_text, 1, wx.EXPAND, 0)
			self.sub_sizer.Add(self.send_button, 0, wx.EXPAND|wx.LEFT, 3)
			
			# Create the main sizer
			self.sizer = wx.BoxSizer(wx.VERTICAL)
			
			# Add the display and sub sizer to the main sizer
			self.sizer.Add(self.disp_text, 1, wx.EXPAND|wx.TOP|wx.RIGHT|wx.LEFT, 4)
			self.sizer.Add(self.sub_sizer, 0, wx.EXPAND|wx.ALL, 4)
			
			# Set the sizer as the windows main sizer
			self.SetSizer(self.sizer)
			
			# Bind Events
			wx.EVT_TEXT_ENTER(self, self.ID_INPUT, self.send_event)
			wx.EVT_BUTTON(self, self.ID_SEND, self.send_event)
			
			# Set the inital display value
			self.disp_text.SetValue("Please press send to initiate the service menu.")
			self.input_text.Enable(False)
			
		def send_event(self, event):
			"""Service menu send event, fires on enter in the input box
			or when the send button is pressed"""
			
			# Enable the input text
			self.input_text.Enable(True)
			
			# Get the input text
			input_text = self.input_text.GetValue()
			
			# Check to see if the input is blank
			if input_text == "":
				# Set the input to dial the service number
				input_text = "#100#"
				
				
			# Send the serial message
			if not modem_tty == None:
				modem_tty.send(SC_DialString(input_text))
			
			# Clear the input text
			self.input_text.ChangeValue("")
			
		def update_display(self, data):
			"""Updates the display text control"""
			# Check to see if data was provided (empty indicates the
			#	menu has closed)
			if data == "":
				self.disp_text.SetValue("Service menu closed.\nPlease press send to reinitiate the service menu.")
			else:
				# Change the display text
				self.disp_text.ChangeValue(data)
			
		def serial_connected(self):
			"""Called when the serial port is connected, enables
			input box and send button"""
			
			self.input_text.Enable(True)
			self.send_button.Enable(True)
			
		def serial_disconnected(self):
			"""Called when the serial port is disconnected, disables
			input box and send button"""
			
			self.input_text.Enable(False)
			self.send_button.Enable(False)
			
	class SMSPanel(wx.Panel):
		def __init__(self, parent, id):
			"""Initializes the SMS panel
			
			Parameters:
				parent, Parent window
				id, WX ID
			"""
			
			# Initialize varialbes
			self.msg_list = []
			
			# Initialize the panel object
			wx.Panel.__init__(self, parent, id)
			
			# Create a list control to list SMS messages
			self.display_list = wx.ListCtrl(self, wx.ID_ANY,
				style=wx.LC_REPORT|wx.LC_SORT_ASCENDING|wx.LC_VRULES|wx.BORDER)

			
			# Create new, read, and delete buttons
			self.ID_UPDATE = wx.NewId()
			self.ID_NEW = wx.NewId()
			self.ID_READ = wx.NewId()
			self.ID_DELETE = wx.NewId()
			
			self.update_button = wx.Button(self, self.ID_UPDATE, "Update")
			self.new_button = wx.Button(self, self.ID_NEW, "New")
			self.read_button = wx.Button(self, self.ID_READ, "Read")
			self.delete_button = wx.Button(self, self.ID_DELETE, "Delete")
			
			# Create a sizer to hold the buttons
			self.button_sizer = wx.BoxSizer(wx.HORIZONTAL)
			
			# Add buttons and a spacer to the sizer
			self.button_sizer.Add(self.update_button, 0, wx.EXPAND, 0)
			self.button_sizer.Add(self.new_button, 0, wx.EXPAND|wx.LEFT, 5)
			self.button_sizer.Add(self.read_button, 0, wx.EXPAND|wx.LEFT, 5)
			self.button_sizer.AddStretchSpacer()
			self.button_sizer.Add(self.delete_button, 0, wx.EXPAND, 0)
			
			# Bind button events
			wx.EVT_BUTTON(self, self.ID_UPDATE, self.update_click)
			wx.EVT_BUTTON(self, self.ID_NEW, self.new_click)
			wx.EVT_BUTTON(self, self.ID_READ, self.read_click)
			wx.EVT_BUTTON(self, self.ID_DELETE, self.delete_click)
			
			# Create a sizer for the panel
			self.sizer = wx.BoxSizer(wx.VERTICAL)
			
			# Add display and buttons to the sizer
			self.sizer.Add(self.display_list, 1, wx.EXPAND|wx.ALL, 5)
			self.sizer.Add(self.button_sizer, 0, wx.EXPAND|wx.LEFT|wx.RIGHT|wx.BOTTOM, 5)
			
			# Set the panels sizer
			self.SetSizer(self.sizer)
			
			# Add columns to the list
			self.display_list.InsertColumn(0, "Date")
			self.display_list.InsertColumn(1, "Type")
			self.display_list.InsertColumn(2, "Read")
			self.display_list.InsertColumn(3, "From/To")
		
		def update_click(self, event):
			"""Update button click event"""
			
			# Update the list
			self.active(True)
			
		def new_click(self, event):
			"""New button click event"""
			
			# Open the SMS creation dialog
			JBMSMSDialog(self, wx.ID_ANY)
			
		def read_click(self, event):
			"""Read button click event"""
			# Check to make sure there are message
			if len(self.msg_list) > 0:
				# Open the selected message to read
				JBMSMSDialog(self, wx.ID_ANY, True,
					self.msg_list[self.display_list.GetFirstSelected()])
			
			
		def delete_click(self, event):
			"""Delete button click event"""
			
			# Check to make sure there are message
			if len(self.msg_list) == 0:
				return
				
			# Dialog requesting confirmation...
			ConfDia = wx.MessageDialog(None, 
					"Are you sure you would like to delete the selected message(s)?",
					"Delete Confirmation", wx.OK|wx.CANCEL|wx.ICON_ERROR)
			ConfDia.ShowModal()
			
			# Check the return value
			if ConfDia.GetReturnCode() == wx.CANCEL:
					ConfDia.Destroy()
					return
					
			ConfDia.Destroy()
			
			# Remember all selected indexes
			index_list = []
			
			tmp_index = self.display_list.GetFirstSelected()
			while tmp_index > -1:
				index_list.append(tmp_index)
				tmp_index = self.display_list.GetNextSelected(tmp_index)
			
			# Sort the index in a reverse order
			index_list.sort(reverse=True)
			
			# Loop through recorded list
			for tmp_index in index_list:
				# Delete from the sim
				if not modem_tty == None:
					modem_tty.send(SC_SMS_Delete(self.msg_list[tmp_index].index))
				
				# Remove from the display
				self.display_list.DeleteItem(tmp_index)
				
				# Remove from the message list
				self.msg_list.pop(tmp_index)
				
		def active(self, update=False):
			"""Called when the panel is selected"""
			
			# Check to see if SMS has been initated and if the 
			#	SMS list has been populated
			if (wx_application.sms_init and len(self.msg_list) == 0) or update:
				# Check to see if the modem is valid
				if not modem_tty == None:
					# Request a list of messages from the modem
					modem_tty.send(SC_SMS_List("ALL", self.sms_list_callback))
					return
			
			if len(self.msg_list) == 0:
				# Clear the display
				self.display_list.DeleteAllItems()
			
				# Indicate that the modem isn't ready
				self.display_list.Append(["", "Modem not found or not yet initialized", "", ""])
				self.display_list.Append(["", "Please try again later", "", ""])
				
				# Update column widths
				self.display_list.SetColumnWidth(0, 0)
				self.display_list.SetColumnWidth(1, wx.LIST_AUTOSIZE)
				self.display_list.SetColumnWidth(2, 0)
				self.display_list.SetColumnWidth(3, 0)
		
		def sms_list_callback(self, msg):
			"""Called when a list of SMS messages is returned from the modem"""
			
			# Clear the display
			self.display_list.DeleteAllItems()
			
			# Check for errors
			if msg.type == JBM_MSG_SER_ERROR or msg.type == JBM_MSG_SER_TIMEOUT:
				# Update the list to show the error
				self.display_list.Append(["", "Error updating message list, trying again", "", ""])
			
				# Update column widths
				self.display_list.SetColumnWidth(0, 0)
				self.display_list.SetColumnWidth(1, wx.LIST_AUTOSIZE)
				self.display_list.SetColumnWidth(2, 0)
				self.display_list.SetColumnWidth(3, 0)
				
				# Re-init sms
				modem_tty.send(SC_SMS_Init(config.get("sc number","general","+61418706700")))
					
				# Return true so that JBM will process the message
				return True
				
			# Remember the list
			self.msg_list = msg.data
			
			# Update the display
			for msg in self.msg_list:
				if (msg.flags&SMS_RECIEVED) == 0:
					type = "Stored"
				else:
					type = "Recieved"
					
				if (msg.flags&SMS_READ) == 0:
					read = "Unread"
				else:
					read = "Read"
					
				self.display_list.Append([msg.datestamp, type, read, msg.number])
			
			# Update column widths
			self.display_list.SetColumnWidth(0, wx.LIST_AUTOSIZE)
			self.display_list.SetColumnWidth(1, wx.LIST_AUTOSIZE)
			self.display_list.SetColumnWidth(2, wx.LIST_AUTOSIZE)
			self.display_list.SetColumnWidth(3, wx.LIST_AUTOSIZE)
			
			# Return false indicating no further processing required
			return False
			
	class OptionsPanel(wx.Panel):
		
		class OptionDialog(wx.Dialog):
			"""Window for editing/adding config settings"""
			
			def __init__(self, parent, id, section="", key="",
					value="", type="", update=None):
				"""Initializes the dialog
				
				Parameters:
					parent, window parent
					
					id, window id
					
					section, setting section
					
					key, setting key
					
					value, setting value
					
					type, setting type
					
					update, callback used if a setting is changed, takes
							no parameters
							
					Note: Unless blank, section, key, and type are readonly
				"""
				
				# Remember the update callback
				self.update = update
				
				# Make title, default attributes and help text
				if section == "" and key == "":
					title = "New Setting"
					help_text = "Creating a new setting, setting should be in the form section.key\n"
					attributes = 0
					section_text = ""
				else:
					title = "Edit Setting [{0}.{1}]".format(section, key)
					help_text = "To return setting to default value delete it\n".format(section, key)
					attributes = wx.TE_READONLY
					section_text = "{0}.{1}".format(section, key)
				
				# Initialize the frame
				wx.Dialog.__init__(self, parent, id, title,
						size=(400,200),
						style=wx.DEFAULT_DIALOG_STYLE|wx.DIALOG_MODELESS)

				# Create a static text to show help message
				self.help_text = wx.StaticText(self, wx.ID_ANY,
						help_text + "A restart is required for most setting changes to take place")
				
				# Create Section/key Display
				self.sect_label = wx.StaticText(self, wx.ID_ANY, "Setting:")
				self.sect_text = wx.TextCtrl(self, wx.ID_ANY, section_text, style=attributes)
				
				# Creat a sizer to hold section/key
				self.sect_sizer = wx.BoxSizer(wx.HORIZONTAL)
				
				# Add section/key to sizer
				self.sect_sizer.Add(self.sect_label, 0, wx.EXPAND, 0)
				self.sect_sizer.Add(self.sect_text, 1, wx.EXPAND|wx.LEFT, 2)
				
				# Create Value Display
				self.value_label = wx.StaticText(self, wx.ID_ANY, "Value:")
				self.value_text = wx.TextCtrl(self, wx.ID_ANY, value)
				
				# Create Type Display
				self.type_text = wx.ComboBox(self, wx.ID_ANY, type, style=attributes)
				
				# Add values to the Combo box
				self.type_text.Append("int")
				self.type_text.Append("float")
				self.type_text.Append("bool")
				self.type_text.Append("str")
				
				# Disable the combobox if in edit mode
				if not attributes == 0:
					self.type_text.Enable(False)
					
				# Create a sizer to hold the value and type
				self.value_sizer = wx.BoxSizer(wx.HORIZONTAL)
				
				# Add value and type
				self.value_sizer.Add(self.value_label, 0, wx.EXPAND, 0)
				self.value_sizer.Add(self.value_text, 1, wx.EXPAND|wx.LEFT, 2)
				self.value_sizer.Add(self.type_text, 0, wx.EXPAND|wx.LEFT, 2)
				
				# Create Ok and Cancel buttons
				self.ID_OK = wx.NewId()
				self.ID_CANCEL = wx.NewId()
				
				self.ok_button = wx.Button(self, self.ID_OK, "Ok")
				self.cancel_button = wx.Button(self, self.ID_CANCEL, "Cancel")
				
				# Bind Events
				wx.EVT_BUTTON(self, self.ID_OK, self.ok_click)
				wx.EVT_BUTTON(self, self.ID_CANCEL, self.cancel_click)
				
				# Create a sizer to hold the buttons
				self.button_sizer = wx.BoxSizer(wx.HORIZONTAL)
				
				# Add a spacer
				self.button_sizer.AddStretchSpacer()
				
				# Add buttons
				self.button_sizer.Add(self.ok_button, 0, wx.EXPAND|wx.RIGHT, 5)
				self.button_sizer.Add(self.cancel_button, 0, wx.EXPAND, 0)
			
				# Creat a the main panel sizer
				self.sizer = wx.BoxSizer(wx.VERTICAL)
				
				# Add everything to the main sizer
				self.sizer.Add(self.help_text,1, wx.EXPAND|wx.ALL, 5)
				self.sizer.Add(self.sect_sizer, 0, wx.EXPAND|wx.RIGHT|wx.LEFT|wx.BOTTOM, 5)
				self.sizer.Add(self.value_sizer, 0, wx.EXPAND|wx.RIGHT|wx.LEFT|wx.BOTTOM, 5)
				self.sizer.Add(self.button_sizer, 0, wx.EXPAND|wx.RIGHT|wx.LEFT|wx.BOTTOM, 5)
				
				# Set the sizer
				self.SetSizer(self.sizer)
				
				# Show the window
				self.ShowModal()
			
			def ok_click(self, event):
				"""OK Button was clicked"""
				
				# Paras the section/key value
				try:
					tmp = self.sect_text.GetValue()
					(sect, key) = tmp.split(".")
				except ValueError:
					sect = ""
					key = ""
					
				# Verify the section and key values
				if sect == "" or key == "":
					error_dia = wx.MessageDialog(None, "Setting must take the form of <section>.<setting>",
						"Error", wx.OK | wx.ICON_ERROR)
					error_dia.ShowModal()
					error_dia.Destroy()
					
					print "Error, Invalid Section/Key Value"
					return
					
				# Process the value and type
				value = self.value_text.GetValue()
				type = self.type_text.GetValue()
				
				try:
					if type == "int":
						value = int(value)
					elif type == "float":
						value = float(value)
					elif type == "bool":
						value = value.lower()
						if value == "true" or value == "yes":
							value = True
						else:
							value = False
					else:
						value = str(value)
						
				except ValueError:
					# Could not process the value
					
					error_dia = wx.MessageDialog(None, "Invalid value for type {0}".format(type),
						"Error", wx.OK | wx.ICON_ERROR)
					error_dia.ShowModal()
					error_dia.Destroy()
					
					print "Error, Invalid Value"
					return
					
				# Set the setting and tell the config panel to update
				config.set(key, value, sect)
				
				# Update the config panel
				self.update()
				
				# Destroy the window
				self.Destroy()
				
			def cancel_click(self, event):
				"""Cancel Button was clicked"""
				
				# Destroy the window
				self.Destroy()
								
		
		def __init__(self, parent, id):
			"""Initiates the options panel
			
			Parameters:
				parent, Parent window
				id, WX ID
			"""
			
			# Initiate the panel object
			wx.Panel.__init__(self, parent, id)		
	
			# Create a sizer for the panel
			self.sizer = wx.BoxSizer(wx.VERTICAL)
			
			# Create the main list view
			self.list = wx.ListCtrl(self, wx.ID_ANY,
				style=wx.LC_REPORT|wx.LC_SORT_ASCENDING|wx.LC_VRULES|wx.BORDER|wx.LC_SINGLE_SEL)
				
			self.sizer.Add(self.list, 1, wx.EXPAND|wx.ALL, 5)
			
			# Create a sizer to hold the buttons
			self.button_sizer = wx.BoxSizer(wx.HORIZONTAL)
		
			# Create the new, edit and delete buttons
			self.ID_NEW = wx.NewId()
			self.ID_EDIT = wx.NewId()
			self.ID_DELETE = wx.NewId()
			
			self.new_button = wx.Button(self, self.ID_NEW, "New")
			self.edit_button = wx.Button(self, self.ID_EDIT, "Edit")
			self.delete_button = wx.Button(self, self.ID_DELETE, "Delete")
			
			# Bind the button events
			wx.EVT_BUTTON(self, self.ID_NEW, self.new_click)
			wx.EVT_BUTTON(self, self.ID_EDIT, self.edit_click)
			wx.EVT_BUTTON(self, self.ID_DELETE, self.delete_click)
			
			# Add the buttons and spacers to the sizer
			self.button_sizer.Add(self.new_button, 0, wx.EXPAND|wx.RIGHT, 5)
			self.button_sizer.Add(self.edit_button, 0, wx.EXPAND, 0)
			self.button_sizer.AddStretchSpacer()
			self.button_sizer.Add(self.delete_button, 0, wx.EXPAND|wx.LEFT)
			
			# Add the button sizer to the main sizer
			self.sizer.Add(self.button_sizer, 0, wx.EXPAND|wx.ALL, 5)
			
			# Set the sizer
			self.SetSizer(self.sizer)
			
			# Insert List Columns
			self.list.InsertColumn(0, "Section")
			self.list.InsertColumn(1, "Setting")
			self.list.InsertColumn(2, "Value")
			self.list.InsertColumn(3, "Type")
			
			# Update the settings
			self.refresh()
			
		def refresh(self):
			"""Refreshes the settings list"""
			
			# Clear the list
			self.list.DeleteAllItems()
			
			# Add new items to the list
			for section, key, value, type in config:
				self.list.Append([section, key, value, type])
				
			# Update column widths
			self.list.SetColumnWidth(0, wx.LIST_AUTOSIZE)
			self.list.SetColumnWidth(1, wx.LIST_AUTOSIZE)
			self.list.SetColumnWidth(2, wx.LIST_AUTOSIZE)
			self.list.SetColumnWidth(3, wx.LIST_AUTOSIZE)
			
			
			
		def delete_click(self, event):
			"""Delete button was clicked..."""
			
			# Get the index of the selected item
			tmp_index = self.list.GetFirstSelected()
			
			# Get the section name and key
			tmp_sect = self.list.GetItem(tmp_index, 0)
			tmp_set = self.list.GetItem(tmp_index, 1)
			
			config.remove(tmp_set.GetText(), tmp_sect.GetText())
			
			# Remove the list item
			self.list.DeleteItem(tmp_index)
		
		def new_click(self, event):
			"""New button was clicked"""
			
			# Show new window
			JBMMainWindow.OptionsPanel.OptionDialog(self, wx.ID_ANY,
					update=self.refresh)
			
			
		def edit_click(self, event):
			"""Edit button was clicked..."""
			
			# Get the index of the selected item
			tmp_index = self.list.GetFirstSelected()
			
			# Get the section name and key
			tmp_sect = self.list.GetItem(tmp_index, 0)
			tmp_set = self.list.GetItem(tmp_index, 1)
			tmp_value = self.list.GetItem(tmp_index, 2)
			tmp_type = self.list.GetItem(tmp_index, 3)
			
			# Show edit window
			JBMMainWindow.OptionsPanel.OptionDialog(self, wx.ID_ANY,
				tmp_sect.GetText(), tmp_set.GetText(), tmp_value.GetText(),
				tmp_type.GetText(), self.refresh)
			
			
			
	class AboutPanel(wx.Panel):
		def __init__(self, parent, id):
			"""Initiates the about panel
			
			Parameters:
				parent, Parent window
				id, WX ID
			"""
			# Initiate variables
			self.modem_info = {}
			
			# Initiate the panel object
			wx.Panel.__init__(self, parent, id)
			
			# Create a text control to display all of the about info
			self.disp_text = wx.TextCtrl(self, wx.ID_ANY,
				style=wx.TE_MULTILINE|wx.TE_RICH2|wx.TE_READONLY)
		
			# Create a sizer to hold the text control
			self.sizer = wx.BoxSizer(wx.VERTICAL)
			
			# Add the display to the sizer
			self.sizer.Add(self.disp_text, 1, wx.EXPAND|wx.ALL, 5)
	
			# Set the panels sizer
			self.SetSizer(self.sizer)
			
			# Update the display
			self.update()
			
		def active(self):
			"""Called when page becomes active"""
			
			# Check to see if modem info has been set
			if len(self.modem_info) == 0:
				if not modem_tty == None:
					# Request info
					modem_tty.send(SC_Info(self.info_callback))
		
		def info_callback(self, msg):
			"""Callback for modem information command"""
			
			# Remember the modems information
			self.modem_info = msg.data
			
			# Update the display
			self.update()
		
			# Return False to indicate no further processing required
			return False
			
		def update(self):
			"""Updates the about dialog"""
			self.disp_text.SetValue("""Jon's Broadband Manager
Copyright 2009 Jon Estey
       
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License V2 as published by the Free Software Foundation

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

Modem Information:
""")
			
			
			# Try to show modem information
			try:
				self.disp_text.AppendText("\nManufacturer: {0}".format(self.modem_info["GMI"]))
			except KeyError:
				pass
			
			try:
				self.disp_text.AppendText("\nModel: {0}".format(self.modem_info["GMM"]))
			except KeyError:
				pass
				
			try:
				self.disp_text.AppendText("\nRevision: {0}".format(self.modem_info["GMR"]))
			except KeyError:
				pass
			
			try:
				self.disp_text.AppendText("\nSerial Number: {0}".format(self.modem_info["GSN"]))
			except KeyError:
				pass
				
			try:
				self.disp_text.AppendText("\nIMI Number: {0}".format(self.modem_info["CIMI"]))
			except KeyError:
				pass
				
	def __init__(self):
		"""Initiates the primary display window
		
		Parameters:
			None so far...	
		"""
		
		# Initiate the frame
		wx.Frame.__init__(self, None, wx.ID_ANY,
			"Jon's Broadband Manager", size=(500,300))#,
			#style=wx.DEFAULT_DIALOG_STYLE)
			
		# Create a notebook to hold all of the main sections
		self.notebook = wx.Notebook(self, wx.ID_ANY)
		
		# Create the main panels
		self.connection_panel = JBMMainWindow.ConnectionPanel(
			self.notebook, wx.ID_ANY)
			
		self.usage_panel = JBMMainWindow.UsagePanel(self.notebook,
			wx.ID_ANY)
			
		self.service_panel = JBMMainWindow.ServicePanel(
			self.notebook, wx.ID_ANY)
		
		self.operators_panel = JBMMainWindow.OperatorsPanel(
			self.notebook, wx.ID_ANY)
			
		self.sms_panel = JBMMainWindow.SMSPanel(
			self.notebook, wx.ID_ANY)
		
		self.options_panel = JBMMainWindow.OptionsPanel(
			self.notebook, wx.ID_ANY)
			
		self.about_panel = JBMMainWindow.AboutPanel(self.notebook,
			wx.ID_ANY)
		
		# Add the main panels to the notebook
		self.notebook.AddPage(self.connection_panel, "Connection")
		self.notebook.AddPage(self.usage_panel, "Usage Log")
		self.notebook.AddPage(self.service_panel, "Service Menu")
		self.notebook.AddPage(self.sms_panel, "SMS")
		self.notebook.AddPage(self.operators_panel, "Operators")
		self.notebook.AddPage(self.options_panel, "Options")
		self.notebook.AddPage(self.about_panel, "About")
		self.notebook.SetPageSize((300,500))
		
		# Setup Events
		
		# Window close
		self.Bind(wx.EVT_CLOSE, self.window_closed)
		self.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, self.page_changed)
		
		# Create and update timer
		self.TIMER_ID = wx.NewId()
		self.timer = wx.Timer(self, self.TIMER_ID)
		wx.EVT_TIMER(self, self.TIMER_ID, self.on_timer)
		
		# Show the window
		self.Show(True)
	
	def page_changed(self, event):
		"""Event that fires when the notebook page has changed"""
		
		# Get the current notebook page
		cur_page = self.notebook.GetCurrentPage()
		
		# Trigger the pages acive event
		try:
			cur_page.active()
		except AttributeError:
			pass
			
			
	def on_timer(self, event):
		"""Event for updating various things periodically"""

		# Update the usage
		self.usage_panel.update_list()
		
		# Request signal strength...
		modem_tty.send(SC_SignalStrength(self.signal_callback))
		
		# Stop the timer
		self.timer.Stop()
		
	def signal_callback(self, msg):
		"""Callback for signal strength updates"""
		
		# Check for errors
		if not msg.type == JBM_MSG_SER_ERROR and not msg.type == JBM_MSG_SER_TIMEOUT:	
			# Remember the signal strength
			self.signal = int(msg.data*100)
			
			# Update the signal strength display
			
			# Update the title bar
			if not wx_application.network == "":
				self.SetTitle("Jon's Broadband Manager ({0} {1})".format(wx_application.network, self.signal))
				
				# If the network hasn't come up after there is a signal
				#	ask the modem for it
				if self.signal > 0:
					pass
			else:
				self.SetTitle("Jon's Broadband Manager ({0})".format(self.signal))
			
		# Restart the timer
		self.timer.Start(2500)
		
		return True
			
	def disconnected(self):
		"""Called when the connection is disconnected"""
		
		# Tell the usage panel the connection was closed
		self.usage_panel.disconnected()
		
		# Update the connect button
		self.connection_panel.connect_button.SetLabel("Connect")
		
		# Clear the connection details
		self.connection_panel.ip_disp.SetValue("")
		self.connection_panel.remip_disp.SetValue("")
		self.connection_panel.dns1_disp.SetValue("")
		self.connection_panel.dns2_disp.SetValue("")
		
	def connected(self, interface):
		"""Called when the connection is connected"""
		
		# Tell the usage panel the connection was connect
		self.usage_panel.connected(interface)
		
		# Update the connect button
		self.connection_panel.connect_button.SetLabel("Disconnect")
	
	def serial_connected(self):
		# Start the update timer
		self.timer.Start(2500)
		
		# Tell the service and sms panels that the serial connection is
		#	connected
		self.service_panel.serial_connected()
		#self.sms_panel.serial_connected()
			
	def serial_disconnected(self):
		# Stop the update timer
		self.timer.Stop()
		
		# Update the window title
		self.SetTitle("Jon's Broadband Manager")
		
		# Tell the service and sms panels that the serial connection is
		#	disconnected
		self.service_panel.serial_disconnected()
		#self.sms_panel.serial_connected()
		
	def window_closed(self, event):
		"""Event fired when the window is close"""
		
		# Kill the timer
		self.timer.Stop()
		
		# Disconnect the modem
		if not dialer == None:
			dialer.disconnect()
		
		# Disconnect the connection
		self.disconnected()
		
		# Tell the application to close
		wx_application.keepgoing = False
		
		# Destroy the window
		self.Destroy()
		
		# Continue on wit the default processing
		event.Skip()
		
		


class JBMApplication(wx.App):
	"""Main application object, implements a custom MainLoop to processes
	inter-thread communications, Mostly borrowed from the main loop example
	at...
	http://cvs.wxwidgets.org/viewcvs.cgi/wxWidgets/wxPython/samples/mainloop/mainloop.py?rev=1.1&content-type=text/vnd.viewcvs-markup
	"""
	def __init__(self):
		"""Initiates the application class"""
		
		# Initiate wx.App parent
		wx.App.__init__(self, False)
		
		
	def MainLoop(self):
		"""Processes wx events while also processing inter-threed 
		communications"""
		
		# Create a new event loop
		eventloop = wx.EventLoop()
		
		# Set the new loop to be active
		wx.EventLoop.SetActive(eventloop)

		# Loop while the application should still be running
		while self.keepgoing:
			# Check message queue
			try:
				# Get the next message
				msg = message_queue.get(False)
			except Queue.Empty:
				pass
			else:
				# Process the message
				self.ProcessMessage(msg)
				
				# Tell queue processing is done
				message_queue.task_done()
				
			# Process GUI events
			while eventloop.Pending():
				eventloop.Dispatch()
			
			# Sleep a bit.. This may need to be removed/moved
			sleep(0.10)
			
			# Process idle messages
			self.ProcessIdle()
		
		# Kill the serial connection
		if not modem_tty == None:
			modem_tty.close()
			
		# Kill the detect timer if present
		if not self.detect_timer == None:
			self.detect_timer.Stop();
			del self.detect_timer
			self.detect_timer = None;
	
	def ProcessMessage(self, msg):
		"""Processes inter-thread messages"""
		global modem_tty
		
		# Check for a callback
		if not msg.callback == None:
			if not msg.callback(msg):
				return
				
		# Dialer Messages
		if msg.type == JBM_MSG_CONNECT:
			# Tell the main window the connection is opened
			self.mainwindow.connected(msg.data)
			
		elif msg.type == JBM_MSG_DISCONNECT:
			# Tell the main window the connection is closed
			self.mainwindow.disconnected()
			
			# Clear resolv.conf if required
			if config.get("set nameserver","general",True):
				try:
					file = open("/etc/resolv.conf","w")
					file.write("# Generated by Jon's Broadband Manager\n".format(msg.data))
				except IOError:
					# Error??
					print "Error writing to resolve.conf"
			
		elif msg.type == JBM_MSG_DNS1:
			# Set the dns display...
			self.mainwindow.connection_panel.dns1_disp.ChangeValue(msg.data)
			
			# Replace resolve.conf if required
			if config.get("set nameserver","general",True):
				try:
					file = open("/etc/resolv.conf","w")
					file.write("# Generated by Jon's Broadband Manager\nnameserver {0}\n".format(msg.data))
				except IOError:
					# Error??
					print "Error writing to resolve.conf"
								
		elif msg.type == JBM_MSG_DNS2:
			# Set the secondary dns display...
			self.mainwindow.connection_panel.dns2_disp.ChangeValue(msg.data)
			
			# Add to resolve.conf if required
			if config.get("set nameserver","general",True):
				try:
					file = open("/etc/resolv.conf","a")
					file.write("nameserver {0}\n".format(msg.data))
				except IOError:
					# Error??
					print "Error writing to resolve.conf"
					pass
		elif msg.type == JBM_MSG_IP:
			# Set the ip display...
			self.mainwindow.connection_panel.ip_disp.ChangeValue(msg.data)
		elif msg.type == JBM_MSG_REM_IP:
			# Set the remote ip display...
			self.mainwindow.connection_panel.remip_disp.ChangeValue(msg.data)
			
		# Serial Messages
		elif msg.type == JBM_MSG_SER_CLOSED:
			# Serial port is closed, disable update timer, service menu,
			#	and sms
			self.mainwindow.serial_disconnected()
			
			# Clear sms init flags
			self.sms_init = False
			self.sms_init_wait = 0
			
			# Delete the modem object and start the scan timer
			del modem_tty
			modem_tty = None
			
			
			# Start the scan timer
			DETECT_TIMER_ID = wx.NewId()
			self.detect_timer = wx.Timer(self, DETECT_TIMER_ID)
			wx.EVT_TIMER(self, DETECT_TIMER_ID, self.modem_detect_timer)
			self.detect_timer.Start(5000)
			
		elif msg.type == JBM_MSG_SER_OPEN:
			# Serial port is opened, enable update timer, and service menu
			self.mainwindow.serial_connected()
			
			#modem_tty.send(SC_CDMode())
		elif msg.type == JBM_MSG_SER_ERROR:
			# There was an serial error report it
			print "Serial Error: {0}".format(msg.data)
			
		elif msg.type == JBM_MSG_NETWORK:
			# Remember the network
			self.network = msg.data
		elif msg.type == JBM_MSG_SIGNAL:
			# Check to see if SMS has been intiated and if there is a
			#	signal
			if not self.sms_init and msg.data > 0:
				# Check to see if an init was already sent and hasn't
				#	expired
				if self.sms_init_wait > 0 and self.sms_init_wait < 12:
					# Increment the wait timer
					self.sms_init_wait += 1
				else:
					# Initiate SMS
					modem_tty.send(SC_SMS_Init(config.get("sc number","general","+61418706700")))
					
					# Reset the wait timer
					self.sms_init_wait = 1
		elif msg.type == JBM_MSG_CUSD:
			# Update the service menu display
			self.mainwindow.service_panel.update_display(msg.data)
		elif msg.type == JBM_MSG_SMS_READY:	
			# Indicate that SMS has been initated
			self.sms_init = True
		elif msg.type == JBM_MSG_SMS_SENT:
			# SMS Message has been sent
			pass
		elif msg.type == JBM_MSG_SMS:
			# SMS is trying to be read, open it up
			
			# Open the SMS
			JBMSMSDialog(self.mainwindow, -1, True, msg.data)
		elif msg.type == JBM_MSG_SMS_REC:
			# Tell the SMS list to refresh
			self.mainwindow.sms_panel.active(True)
			
			# New SMS Message has arrived, open it up if required
			if config.get("auto open", "sms", True):
				modem_tty.send(SC_SMS_Read(msg.data))	
			elif config.get("notify on new","sms",True):
				NotifyDia = wx.MessageDialog(None, 
						"A New SMS Message has arrived, would you like to open it?",
						"New SMS", wx.YES | wx.NO | wx.ICON_INFORMATION)

				# Check the return code and open the message if requested
				if NotifyDia.ShowModal() == wx.ID_YES:
					modem_tty.send(SC_SMS_Read(msg.data))	
				
				NotifyDia.Destroy()
				
			# Play a sound for the new sms
		elif msg.type == JBM_MSG_CDMODE:
			# Print state
			if msg.data:
				print "Modem CD Mode Enabled"
			else:
				print "Modem CD Mode Disabled"
		
	def OnInit(self):
		"""Creates the main window, the dialer, and the serial connection"""
		
		global dialer
		global modem_tty
		global modem_tty_port
		global config
		global message_queue
		
		# Set the keepgoing flag
		self.keepgoing = True
		
		# Load the configuration
		config = JBMConfig(config_path, verbosity)

		# Create the message queue
		message_queue = Queue.Queue()

		# Create the main window
		self.mainwindow = JBMMainWindow()
		self.SetTopWindow(self.mainwindow)
		
		self.detect_timer = None
		
		# Try to configure the modem
		if not self.modem_config():
			# Create a slow timer to monitor for a modem connection
			DETECT_TIMER_ID = wx.NewId()
			self.detect_timer = wx.Timer(self, DETECT_TIMER_ID)
			wx.EVT_TIMER(self, DETECT_TIMER_ID, self.modem_detect_timer)
			self.detect_timer.Start(5000)
			
		else:
			# Create the tty connection
			modem_tty = JBMSerial(message_queue, config)

			# Open the tty connection
			modem_tty.start()
			
		# Initially there is no network connection and signal strength
		#	is zero
		self.network = ""
		self.signal = 0
		
		# Set sms flags
		self.sms_init = False
		self.sms_init_wait = 0
		
		# Return true to continue init process	
		return True
	
	def modem_detect_timer(self, event):
		
		global modem_tty
		
		# Attempt to configure the modem
		if self.modem_config():
			# Kill the timer
			self.detect_timer.Stop();
			del self.detect_timer
			self.detect_timer = None;
			
			# Create the tty connection
			modem_tty = JBMSerial(message_queue, config)

			# Open the tty connection
			modem_tty.start()		
	
	def modem_config(self):
		# Check to see if autodetection should occur
		
		if not config.get("detect tty", "general", True):
			return True;
			
		"""Attempts automatic configuration of the modem"""
		import jbm_detect
		
		# Check for a valid modem if required
		if config.get("check modem", "general", True):
			retval = jbm_detect.usb_detect()
			
			if (retval & jbm_detect.MODEM_MODE) > 0:
				pass
			elif (retval & jbm_detect.CD_MODE) > 0:
				# Should modeswitch be run
				if config.get("auto modeswitch", "general", True):
					print "Modem found in CD Mode, attempting to switch"
					
					# Try to switch the modem to modem mode
					if not jbm_detect.modeswitch(retval):
						# Report the error
						print " Mode switching failed!"
					else:
						print " Mode switching successful!"
				else:	
					# Valid modem found, but its in CD mode run usb_modeswitch
					print "Modem found in CD Mode, please run usb_modeswitch\
							then try again"

				return False
			else:
				# No Valid modem found
				return False
		
		# Try to detect the modem ports
		command_tty, dial_tty = jbm_detect.scan_tty()
		
		# Report an error if the ports weren't found
		if command_tty == None:
			return False
			
		# Save the ports
		config.set("port", command_tty, "serial")
		config.set("port", dial_tty, "dialer")
		
		return True;


# **************** Main ****************

# Initialize globals
config = None
dialer = None
modem_tty = None
message_queue = None

# Process the command line
show_help = False
verbosity = 0
i = 1

while i < len(sys.argv):
	# Check for command line switches
	arg = sys.argv[i] # Just so its easier to type
	
	if not arg[0] == "-":
		print "Unknown command line option {0}".format(arg)
		show_help = True
		break
		
	if arg == "-h" or arg == "--help":
		show_help = True
		break
	elif arg == "-c" or arg == "--config":
		# The next argument should be a new configuration file path
		i += 1
		config_path = sys.argv[i]
		
		# Show the value
		print "New Config Path: {0}".format(config_path)
	elif arg == "-r" or arg == "--redirect":
		# Redirect stdout to the provided file
		i += 1
		
		# Try to open the provided file or the output file if default is
		#	specified
		if sys.argv[i] == "default":
			try:
				file = open(os.path.join(main_path,"output"), "w")
			except:
				print "Fatal error could not open output file:\n\t{0}".format(os.path.join(main_path,"output"))
				exit()
		else:
			try:
				file = open(sys.argv[i], "w")
			except:
				print "Fatal error could not open output file:\n\t{0}".format(sys.argv[i])
				exit()
		

		# Print message indicating redirection
		print "Redirecting Output to {0}".format(file.name)
		
		# Redirect output
		sys.stdout = file
			
	elif arg == "-u" or arg == "--usage":
		# The next argument should be a new usage log path
		i += 1
		usage_log_path = sys.argv[i]
		
		# Show the value
		print "New Usage Log Path: {0}".format(usage_log_path)
	elif arg == "-w" or arg == "--wvdial":
		# The next argument should be a new wvdial rc path
		i += 1
		#wvdial_path = sys.argv[i]
		
		# Show the value
		print "New WVDial Config Path: {0}".format(wvdial_path)
	elif arg == "-v" or arg == "--verbosity":
		# The next argument is the verbosity
		i += 1
		verbosity = int(sys.argv[i])
		
		# Show the value
		print "Verbosity set to {0}".format(verbosity)
	else:
		print "Unknown command line option {0}".format(arg)
		show_help = True
		break
		
	# Move on to the next argument
	i += 1
	
# Check to see if help should be shown
if show_help:
	# Show the help
	print """
Jon's Broadband Manager 
Writen by Jon Estey

Usage: jbm [-h|--help] [-c|--config <file>] [-r|--redirect <file>]
		[-u|--usage <file>] [-w|--wvdial <file>] [-v|--verbosity <value>]

Arguments:
	-h,--help	Shows this message
	
	-c,--config	Sets the configuration file to <file>
	
	-r,--redirect	Redirects output to <file>
			if <file> is "default" output is redirected to
			~/.tbm/output
			*Please note the output file will be replaced after each
			run.
	
	-u,--usage	Sets the usage log to <file>
	
	*** Commands below this point are currently disabled ***
	-w,--wvdial	Sets the wvdial configuration file to <file>
			*Please note that the wvdial configuration file maybe be
			modified without warning!
	
	-v,--verbosity	Sets the applications verbosity level
			<value> should be one of:
			0 -	Quiet, minimal output
			1 - Talkative Dialer, outputs dialer messages
			2 - Talkative Serial, outputs serial messages
			4 - Talkative Config, outputs config messages
			
			Values can be added together for even more verbose
			output. eg. 5 would show dialer and config messages
"""
	# Exit the program
	exit()

# Check to see if the main directory exists, create it if required
if not os.path.isdir(main_path):
	os.makedirs(main_path)

# Try to open the pid file
pid_file = open(pid_path, 'w')
try:
	# Try to lock the pid file
    fcntl.lockf(pid_file, fcntl.LOCK_EX | fcntl.LOCK_NB)
except IOError:
    # Another instance must be already running
	
	# Print an error message
	print "The program is already running!"
	
	# Exit the program
	sys.exit(0)

# Create the application object
wx_application = JBMApplication()

# Enter the main app loop
wx_application.MainLoop()

# Delete the application object
del wx_application

# Save the config
config.save()
