#!/usr/bin/python

#
# File:	jbm_config.py
#
# Description: Configuration manager for Jon's Broadband Manager
#
# License:
#		jbm_config.py
#		Copyright 2009 Jon Estey
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License version 2 as
#		published by the Free Software Foundation
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#


# **************** Import ****************
import os
from xml.etree import ElementTree
from threading import RLock

# **************** Classes ****************

class JBMConfig():
	"""Main Class for handling configuration options"""
	
	def __init__(self, file, verbosity=0, no_load=False):
		"""Initializes the class
		
		Parameters:
		file, Filename and path of the configuration file
		no_load, if True the configuration file is not loaded at startup
		"""
		
		# Remember the file name
		self.filename = file
		
		# Initiate class variables
		self.config = ElementTree.ElementTree()
		self.cur_section = None
		self.cur_section_name = ""
		self.lock = RLock()
		self.verbosity = verbosity
		
		# Load the file unless told not to
		if not no_load:
			self.load()
	
	def __iter__(self):
		"""The config itself is an iterator"""
		
		# Lock the config
		self.lock.acquire()
		
		# Initialize the iteration
		self._iter_sect = self.config.getiterator("section")
		self._iter_sect_index = 0
		self._iter_set = None
		self._iter_set_index = 0
		
		return self
		
	def next(self):
		"""Returns the next item in an iteration"""
		
		if self._iter_sect_index < len(self._iter_sect):
			# Check to see if this is the first time the section has
			#	been iterated
			if self._iter_set == None:
				self._iter_set = self._iter_sect[self._iter_sect_index].getiterator("setting")
				self._iter_set_index = 0
				
			if self._iter_set_index < len(self._iter_set):
				# Gather values
				section = self._iter_sect[self._iter_sect_index].get("name")
				key = self._iter_set[self._iter_set_index].get("name")
				value = self.process_value(self._iter_set[self._iter_set_index])
				type = self._iter_set[self._iter_set_index].get("type")
				
				self._iter_set_index += 1
				
				return section, key, value, type
			else:
				# Section is done
				self._iter_set = None
				self._iter_sect_index += 1
				
				# Return the next value
				return self.next()
		else:
			# Iteration is done, clean up
			self.lock.release()
			raise StopIteration
			
	def load(self):
		"""Loads the configuration file"""
		
		print "Config: Loading -> {0}".format(self.filename)
		
		# Lock the config
		self.lock.acquire()
		
		# Check to see if the file exists
		if not os.path.exists(self.filename):
			# Create a root element
			root_element = ElementTree.Element("config")
			
			# Set the root element
			self.config._setroot(root_element)
			
			# Unlock the config
			self.lock.release()

			return
			
		# Load the configuration
		self.config.parse(self.filename)
		
		# Unlock the config
		self.lock.release()
		
	def save(self):
		"""Saves the configuration file"""
		
		print "Config: Saving -> {0}".format(self.filename)
		
		# Lock the config
		self.lock.acquire()
		
		# Write the configuration
		self.config.write(self.filename)
	
		# Unlock the config
		self.lock.release()
		
	def get_settings(self, section):
		"""Find the section and returns its settings iterator"""
		
		# Lock the config
		self.lock.acquire()
		
		# Get a list of all sections in the config file
		sections = self.config.getiterator("section")
		
		# Search the list
		for i in sections:
			#print i.get("name", "No Name...")
			if i.get("name","") == section:
				# Unlock the config
				self.lock.release()
		
				return i.getiterator("setting")
		
		# Unlock the config
		self.lock.release()
	
		# Return None
		return None
	
	def get_section(self, section, add=False):
		"""Find a section, return its element object, optionally adding
		it if its not found
		
		Parameters:
			section, name of the section
			add, if True the section is added if its not found
		"""
		
		# Lock the config
		self.lock.acquire()
		
		# Get a list of all sections in the config file
		sections = self.config.getiterator("section")
		
		# Search the list
		for i in sections:
			if i.get("name","") == section:
				# Unlock the config
				self.lock.release()
		
				# Return the section
				return i
				
		# Section not found, should it be added
		if not add:
			# Unlock the config
			self.lock.release()
		
			return None
			
		# Get the root element
		root_element = self.config.getroot()
		
		# Create a new section element
		element = ElementTree.Element("section",{"name":section})

		# Append the new element
		root_element.append(element)
		
		# Unlock the config
		self.lock.release()
		
		
		return element
		
	def process_value(self, setting):
		"""Process the setting, returning setting with the proper type"""
		
		# Get the type and value of the setting, default type is string
		type = setting.get("type","str")
		value = setting.get("value","")
		
		# Convert the value from a string to the appropriate value
		if type == "int":
			return int(value)
		elif type == "float":
			return float(value)
		elif type == "bool":
			value = value.lower()
			if value == "true" or value == "yes":
				return True
			else:
				return False
		else:
			# Default to returning the string value
			return value
			
	def get(self, key, section="default", default="", add=True):
		"""Gets a setting from the configuration file
		
		Parameters:
			key, the key of the setting to return
			section, the section the key is found in
			default, value to return if the key doesn't exist
			add, Adds non-existant settings
		"""
		print "Config: Get -> {0}.{1}".format(section, key)
		
		# Lock the config
		self.lock.acquire()
		
		# Check to see if provided section is already loaded
		if not self.cur_section_name == section:
			# Remember the section
			self.cur_section_name = section
			
			# Remember the iterator
			self.cur_settings = self.get_settings(section)
		
		# Check to see if a section was found
		if self.cur_settings == None:
			# Add the setting if required
			if add:
				self.set(key, default, section)
				
			# Unlock the config
			self.lock.release()
			return default
		
		# Iterate through the settings
		for i in self.cur_settings:
			#print "->" + i.get("name", "No name...")
			if i.get("name","") == key:
				# Unlock the config
				self.lock.release()

				# Return the processed value
				return self.process_value(i)
				
		# Unlock the config
		self.lock.release()
		
		# Check to see if the setting should be added
		if add:
			# Add the value
			self.set(key, default, section)
		
		# Setting not found return the default
		return default
	
	def set(self, key, value, section="default"):
		"""Sets a setting value in section"""
		print "Config: Set -> {0}.{1} = {2}".format(section, key, value)
		
		# Lock the config
		self.lock.acquire()
		
		# Get the section element, adding it if required
		sect_element = self.get_section(section, True)
		
		# Try to find the setting
		settings = sect_element.getiterator("setting")
		for i in settings:
			if i.get("name","") == key:
				# Check the settings type
				if i.get("type","") == type(value).__name__:
					# Set the value
					i.set("value",str(value))
				else:
					# Report the error
					print "Config: Error trying to set {0}.{1}, wrong type, found {2}, expected {3}".format(section, key, type(value).__name__, i.get("type",""))
					
				# Unlock the config
				self.lock.release()
				
				return
				
		# Setting not there, add it
		
		# Create the setting element
		setting_element = ElementTree.Element("setting",{"name":key,"value":str(value),"type":type(value).__name__})
		
		# Add it to the section
		sect_element.append(setting_element)
		
		# Unlock the config
		self.lock.release()
	
	def remove(self, key, section):
		"""Delete the specified setting from the configuration"""
		
		print "Config: Remove -> {0}.{1}".format(section, key)
		
		# Lock the configuration
		self.lock.acquire()
		
		# Get the section element
		sect_element = self.get_section(section, False)
		
		# If the section isn't there then there is no need to remove
		#	the key
		if sect_element == None:
			return
		
		# Try to find the setting
		settings = sect_element.getiterator("setting")
		for i in settings:
			if i.get("name","") == key:
				# Found the setting, remove it
				sect_element.remove(i)
				break

		# Unlock the config
		self.lock.release()
