#!/usr/bin/python

#
# File: jbm_queue.py
#
# Description: Module defining inter-thread messages for Jon's Broadband
#				Manager
#
# License:
#		jbm_queue.py
#		Copyright 2009 Jon Estey
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License version 2 as
#		published by the Free Software Foundation
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#

# **************** Definitions ****************

# Define the various messages that can be sent via the messaging queue

# General Messages
JBM_MSG_ERROR 		= 0x0000 # General error
JBM_MSG_UNREC		= 0x0001 # Unreconized message error, data should
							 # be the JBMMessage that was unrecognized
JBM_REQ_KILL		= 0x0100 # Request to imediately kill the thread

# Serial connection messages
JBM_MSG_SER_ERROR	= 0x1000 # Serial Error
JBM_MSG_SER_CLOSED	= 0x1001 # Serial port is closed
JBM_MSG_SER_OPEN	= 0x1002 # Serial port is open
JBM_MSG_SER_TIMEOUT	= 0x1003 # Serial command timed out
JBM_MSG_NETWORK		= 0x1004 # Modem is connected to the network
JBM_MSG_SIGNAL 		= 0x1005 # Signal strength update
JBM_MSG_CUSD	 	= 0x1006 # CUSD Response
JBM_MSG_SMS_READY	= 0x1007 # SMS has been initiated
JBM_MSG_SMS_SENT	= 0x1008 # SMS Sent
JBM_MSG_SMS_REC		= 0x1009 # New SMS recieved
JBM_MSG_SMS			= 0x100A # Message contains an SMS to be read
JBM_MSG_SMS_LIST	= 0x100B # Message contains a list of SMS
JBM_MSG_INFO		= 0x100C # Modem information
JBM_MSG_CDMODE		= 0x100D # CD Mode state change/inquiry
JBM_MSG_OPLIST		= 0x100E # Operator listing

# Dialer messages
JBM_MSG_DIAL_ERROR	= 0x2000 # Dialer error
JBM_MSG_CONNECT		= 0x2001 # PPP is connected
JBM_MSG_DISCONNECT	= 0x2002 # PPP is disconnected
JBM_MSG_DNS1		= 0x2003 # Primary DNS
JBM_MSG_DNS2		= 0x2004 # Secondary DNS
JBM_MSG_IP			= 0x2005 # Local IP Address
JBM_MSG_REM_IP		= 0x2006 # Remote IP Address

# **************** Classes ****************

class JBMMessage():
	"""Base class for all interthread messages in Jon's Broadband Manager"""
	
	__slots__ = ("type", "data", "callback")
	
	def __init__(self, type, data="", callback=None):
		"""Initates the message class
		
		Parameters:
			type, Message type (See message constants above)
			data, Prudant data for the message, may be any form of data
			callback, Function that is to be called upon processing of
				the message, the callback should be of the form:
	
				def message_callback(message)
	
				Where message will be the message the callback is being
				called from.
				
				Callbacks should return True or nothing if further message
				processing should be preformed, False if the message is
				fully processed
				
				eg. a SC_Signal is sent to the tty containing a callback, the
				command will return a message to the queue containing the
				same callback which will be called by the message processor
			"""
		self.type = type
		self.data = data
		self.callback = callback
		
	
	def __str__(self):
		"""Informal string representation of the message queue"""
		return "JBM Message: {0} Data: <{1}>".format(self.type, str(self.data))
