#!/usr/bin/python

"""
File: jbm_serial.py

Description: The serial control backbone for the Jon's Broadband Manager

License:
	jbm.py
	Copyright 2009 Jon Estey

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License version 2 as
	published by the Free Software Foundation

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	MA 02110-1301, USA.

"""


# **************** Import ****************
import Queue
import serial
import threading
import time
import datetime

from jbm_queue import *

# **************** Definitions ****************
# Command return code flags
SC_PASS			= 0x00  # Response not expected, try the idle processor
SC_ERROR		= 0x01 	# Error occured
SC_EXPECTED		= 0x02 	# Expected responce, don't send to the idle processor
SC_COMPLETE		= 0x04 	# Command is complete move on
SC_NEXT			= 0x08	# This portion of the command is complete, move
						#	on to the next portion
SC_SERVICE		= 0x10	# Flag indicating the service menu was opened
SC_LONG			= 0x20	# Indicates that the command is slow and may need
						# a longer timeout value
						
# SMS Message Status Flags
SMS_RECIEVED	= 0x01 	# Message was recieved
SMS_READ		= 0x02	# Message has been read


# Serial constants
SC_TIMEOUT		= 10.0	# Serial command time out in seconds
SC_TIMEOUT_LONG	= 60.0	# Serial command slow command timeout

# **************** Classes ****************
class SMS():
	"""Class for holding SMS messages"""
	
	__slots__ = ("index", "flags", "number", "message", "datestamp")
	
	def __init__(self, index, flags, number, message, datestamp):
		"""Initializes the class
		
		Parameters:
			flags, Message flags see SMS Message Status Flags
			number, Phone number the message was sent from/to
			message, The message
			datestamp, The time/date when the message was sent/recieved
		"""
		
		self.index = index
		self.flags = flags
		self.number = number
		self.message = message
		self.datestamp = datestamp
		
	
	def __str__(self):
		"""Informal string representation of the SMS message"""
		return "SMS From: {0} Message: \"{1}\"".format(self.number, self.message)

class Operator():
	"""Class that holds information about a given operator"""
	
	__slots__ = ("status", "long_name", "short_name","numeric")
	
	def __init__(self, status=0, long_name="", short_name="", numeric=0):
		"""Initializes the class
		
		Parameters:
			status, Status of the operator
			long_name, Long name of the operator (String)
			short_name, Short name of the operator (String)
			numeric, Numeric name of the operator (Unsigned Long)
		"""
		
		self.status = status
		self.long_name = long_name
		self.short_name = short_name
		self.numeric = numeric
		
	def str_status(self):
		"""Returns a string representation of the operators status"""
		stat_str = ["Unknown","Available","Current","Forbidden"]
		return stat_str[self.status]
		
	
	def __str__(self):
		"""Informal string representation of the operator"""
		return "Operator: {0} Status: \"{1}\"".format(self.long_name, self.str_status())
		
	def list_item(self):
		"""Returns a list of operator parameters, used for creating
		lists controls"""
		return [self.str_status(), self.long_name, self.short_name, self.numeric]
		
class SerialCommand():
	"""Abstract base class for serial commands for the MF626"""		
	
	def GetCommand(self):
		"""Returns the next command that is to be sent to the modem or
		an empty string if the command is done"""
		pass 
		
	def ProcessResponse(self, response, output):
		"""Processes the response from the modem should return...
		
		The default processor only checks standard OK and ERROR responces
		"""
		
		if response == "OK":
			# Got OK, response was expected and command is complete
			return SC_EXPECTED|SC_COMPLETE
		elif response == "ERROR":
			# print "Error Found!"
			# Got Error, report it
			output.put(JBMMessage(JBM_MSG_SER_ERROR, "Serial Error"))
			return SC_EXPECTED|SC_ERROR
			
		return SC_PASS
		
		pass

class SC_Raw(SerialCommand):
	"""Sends a raw command to the serial port, optionally waiting
	for a specific reply"""
	
	def __init__(self, send, expect=""):
		"""Initializer....
		
		Parameters:
			send, data to send to the serial port
			expect, data to wait for, leave blank to wait for standard
				responses
		"""
		
		self.command = 0
		self.send = send
		self.expect = expect
		
	def GetCommand(self):
		self.command += 1
			
		# Check the command index
		if self.command == 1:
			# First command, return data
			return self.send
			
		# No command..
		return ""
		
	def ProcessResponse(self, response, output):
		# Check for echos
		if response == self.send:
			return SC_EXPECTED
		
		# Check to see if there is an expected response
		if self.expect == "":
			# Process standard responce
			return SerialCommand.ProcessResponse(self, response, output)
		
		if response == self.expect:
			return SC_EXPECTED|SC_COMPLETE
			
		return SC_PASS


class SC_CPMS(SerialCommand):
	"""Sends AT+CPMS=? to clear ZUSIMR"""
	
	def __init__(self):
		"""Initializer...."""
		pass
		
	def GetCommand(self):
		# Return the cpms command
		return "AT+CPMS=?"
		
	def ProcessResponse(self, response, output):
		# Check for echos
		if response.startswith("AT+CPMS"):
			return SC_EXPECTED
		elif response.startswith("+CPMS"):
			return SC_EXPECTED
			
		# Process standard responce
		return SerialCommand.ProcessResponse(self, response, output)


class SC_CDMode(SerialCommand):
	"""Serial command to enable/disable cd mode"""
	
	def __init__(self, enable=False, callback=None):
		"""Initializer....
		
		Parameters:
			enable, If True command enables CD Mode, if False cd mode is
				disabled. If None the command checks the current mode
		"""

		# Remember the callback
		self.callback = callback
		
		# Build the command
		if enable == None:
			# Inquiry
			self.command = "AT+ZCDRUN=4"
		elif enable:
			# Enable CD Mode
			self.command = "AT+ZCDRUN=9"
		else:
			# Disable CD Mode
			self.command = "AT+ZCDRUN=8"
					
	def GetCommand(self):
		# Return the command
		return self.command
		
	def ProcessResponse(self, response, output):
		# Check for echos
		if response.startswith("AT+ZCDRUN="):
			return SC_EXPECTED
		
		# Check for status responce
		if response.startswith("Inquiry "):
			output.put(JBMMessage(JBM_MSG_CDMODE,response.endswith("1"),self.callback))
			return SC_EXPECTED
		
		if response.startswith("Close "):
			if response.endswith("1"):
				output.put(JBMMessage(JBM_MSG_CDMODE, False, self.callback))
				return SC_EXPECTED
			else:
				output.put(JBMMessage(JBM_MSG_CDMODE, True, self.callback))
				return SC_EXPECTED
		
		if response.startswith("Open "):
			if response.endswith("1"):
				output.put(JBMMessage(JBM_MSG_CDMODE, True, self.callback))
				return SC_EXPECTED
			else:
				output.put(JBMMessage(JBM_MSG_CDMODE, False, self.callback))
				return SC_EXPECTED	

		# Process standard responce
		return SerialCommand.ProcessResponse(self, response, output)
				
		
class SC_SignalStrength(SerialCommand):
	"""Serial command to get the signal strength"""
	def __init__(self, callback=None):
		"""Initializes the command
		
		Parameters:
			callback, function that will handle the responce from the
						command or none if no callback is required
		"""
		
		self.callback = callback
		
		self.command = 0
		self.found_csq = False
		
	def GetCommand(self):
		self.command += 1
			
		# Check the command index...
		if self.command == 1:
			# Return the signal strength command
			return "AT+CSQ"
		
		# Otherwise return an empty string (only one command)
		return ""
		
	def ProcessResponse(self, response, output):
		# Check to see if CSQ response was already found
		if self.found_csq == True:
			# Process standard responces
			return SerialCommand.ProcessResponse(self, response, output)
		
		# Check for echo
		if response.startswith("AT+CSQ"):
			return SC_EXPECTED
			
		# Check the response
		if response.startswith("+CSQ"):
			# Paras the string
			tmp = response[5:].strip()
			tmp = tmp.split(",")
			tmp = float(tmp[0])
			
			# Check to see if tmp is 99 (unknown)
			if tmp == 99.0:
				tmp = 0
			
			# Calculate signal strength
			sigstr = tmp/31.0
			
			# Output signal strength
			output.put(JBMMessage(JBM_MSG_SIGNAL, sigstr, self.callback))
			
			# Indicate csq was found
			self.found_csq = True
			
			# Return the next signal
			return SC_EXPECTED
		
		# Return pass signal
		return SC_PASS

class SC_Op_Select(SerialCommand):
	"""Serial command to select an operator"""
	
	def __init__(self, numop=None):
		"""Initiatlizes the command
		
		Parameters:
			numop, The numeric name of the operator that is
					to be selected, or None if automatic selection is
					desired
		"""
		
		if numop == None:
			self.command = "AT+COPS=0"
		else:
			self.command = "AT+COPS=1,2,\"{0}\"".format(numop)
	
	def GetCommand(self):
		# Return the command
		return self.command
		
	def ProcessResponse(self, response, output):
		# Check for echos
		if response.startswith("AT+COPS"):
			return SC_EXPECTED
		
		# Process standard responses
		return SerialCommand.ProcessResponse(self, response, output)
		

class SC_Op_List(SerialCommand):
	"""Serial command to list available operators"""
	
	def __init__(self, callback=None):
		"""Initializes the command
		
		Parameters:
			callback, function that will handle the responce from the
						command or none if no callback is required
		"""
		
		self.callback = callback
		self.comnum = 0
		
	def GetCommand(self):
		# Return the command
		if self.comnum == 0:
			return "AT+COPS=?"
		else:
			return "AT+COPS=0"
		
		
	def ProcessResponse(self, response, output):
		# Check for echo
		if response.startswith("AT+COPS"):
			return SC_EXPECTED|SC_LONG
		
		# Check for ZDONR and ZEND
		if response.startswith("+ZDONR") or response.startswith("+ZEND"):
			return SC_EXPECTED|SC_LONG
		
		# Check for return value
		if response.startswith("+COPS"):
			# Paras response
			#+COPS: (3,"YES OPTUS","Optus","50502",0),(3,"vodafone AU","voda AU","50503",),
			# Split up the seperate operators
			operators = response.split("(")
			op_list = []
			
			# Loop through the operators
			for operator in operators:
				# Ignore the first item
				if operator.startswith("+COPS"):
					continue

				# All other operators should be in the form:
				#	<status>, "<long name>", "<short name>", <numerical name>
				
				# Clean up the operator listing
				operator = operator[:-2]
				
				# Split up the operator listing into its sections
				operator = operator.split(",")
				
				i = 0
				
				# The first section is the status
				stat = int(operator[i])
				i += 1
				
				# Long name
				lname = operator[i]
				i += 1
				
				# Name should end in a quote, if it doesn't there was
				#	probably a comma
				while ((not lname.endswith("\"")) and (i < len(operator))):
					lname = lname + "," + operator[i]
					i += 1
				
				# Ensure no problems occured
				if i >= len(operator):
					# Report the error
					print "Serial: Error processing operator, missing \""
					continue
				
				# Trim off quotes
				lname = lname.strip("\"")
				
				# Short name
				sname = operator[i]
				i += 1
				
				# Name should end in a quote, if it doesn't there was
				#	probably a comma
				while ((not sname.endswith("\"")) and (i < len(operator))):
					sname = sname + "," + operator[i]
					i += 1
				
				# Ensure no problems occured
				if i >= len(operator):
					# Report the error
					print "Serial: Error processing operator, missing \""
					continue
				
				# Trim off quotes
				sname = sname.strip("\"")
				
				# Numeric name
				nname = operator[i]
				
				# Trim off quotes
				nname = nname.strip("\"")
				
				# Add the operator to the operators list
				op_list.append(Operator(stat, lname, sname, int(nname)))
				
			# Expected results were found
			output.put(JBMMessage(JBM_MSG_OPLIST, op_list, self.callback))
			return SC_EXPECTED
				
		# Check for ok
		if response.startswith("OK"):
			if self.comnum == 0:
				self.comnum += 1
				return SC_EXPECTED|SC_NEXT
			else:
				return SC_EXPECTED|SC_COMPLETE
		elif response.startswith("ERROR"):
			# Report the error
			return SC_EXPECTED|SC_ERROR
			
		# Return pass signal
		return SC_PASS

class SC_DialString(SerialCommand):
	"""Serial command to dial a string (AT+CUSD)"""
	
	def __init__(self, command):
		"""Initializes the command
		"""
		
		# Initialize variables
		self.command = command
		self.command_sent = False
		self.found_ok = False
		
	def GetCommand(self):
		# Check to see if the command was already sent
		if not self.command_sent:
			# Toggle command sent
			self.command_sent = True
			
			# Return the signal strength command
			return "AT+CUSD=1,\"{0}\",15".format(self.command)
		
		# Otherwise return an empty string (only one command)
		return ""
		
	def ProcessResponse(self, response, output):
		# Check for echo
		if response.startswith("AT+CUSD"):
			return SC_EXPECTED
		
		if not self.found_ok:
			if response == "OK":
				# Ok found, move on
				self.found_ok = True
				return SC_EXPECTED
			elif response == "ERROR":
				# Error found, report it
				return SC_COMPLETE|SC_ERROR
			
			# Pass command to the idle processor
			return SC_PASS
			
		# Check the response
		if response.startswith("+CUSD"):
			# Split the string around the quotes
			data = response.split("\"")
			
			# Put the output data on the output queue
			output.put(JBMMessage(JBM_MSG_CUSD, data[1]))
			
			# Return the next signal
			return SC_EXPECTED|SC_COMPLETE|SC_SERVICE
		
		# Return pass signal
		return SC_PASS

class SC_SMS_Init(SerialCommand):
	"""Serial command to initiate sms communications, setting message
	format and service center number"""
	
	def __init__(self, sc_number):
		"""Initializes the command
		"""
		
		self.sc_number = sc_number
		
		self.sca_sent = False
		self.mgf_sent = False
		self.pms_sent = False
		self.expect_ok = False
			
	def GetCommand(self):
		
		if not self.sca_sent:
			# Set the SCA number
			self.sca_sent = True
			#return "AT+CSCA=\"+61418706700\""
			return "AT+CSCA=\"{0}\"".format(self.sc_number)
			
		if not self.mgf_sent:
			# Set the message format
			self.mgf_sent = True
			return "AT+CMGF=1"
		
		if not self.pms_sent:
			self.pms_sent = True
			return "AT+CPMS=\"ME\",\"ME\",\"ME\""
			
		# No command return a blank
		return ""
		
	def ProcessResponse(self, response, output):
		# Check for echos
		if response.startswith("AT+CSCA") or \
			response.startswith("AT+CMGF") or \
			response.startswith("AT+CPMS"):
			return SC_EXPECTED
		
		# Check for errors
		if response.startswith("+CME"):
			# SIM error, report it
			output.put(JBMMessage(JBM_MSG_SER_ERROR))
			
			return SC_EXPECTED|SC_COMPLETE|SC_ERROR
		
		if response.startswith("+CPMS"):
			# Process the responce
			response = response[7:].split(",")
			
			# Response contains used, total memory for each prefered
			#	memory location, in the order Reading/Deleting,
			#	Writing/Sending, Short Message
			
			return SC_EXPECTED
		
		# Check for ok
		if response == "OK":
			# Check to see if both commands have been sent
			if self.sca_sent and self.mgf_sent and self.pms_sent:
				# Command is done, tell the main thread sms is ready
				output.put(JBMMessage(JBM_MSG_SMS_READY))
				
				return SC_EXPECTED|SC_COMPLETE
			else:
				# Move onto the next command
				return SC_EXPECTED|SC_NEXT
				
		elif response == "ERROR":
			# Error.. doh...
			output.put(JBMMessage(JBM_MSG_SER_ERROR,"Could not initiate SMS"))
			return SC_EXPECTED|SC_COMPLETE|SC_ERROR
			
		# Not expected, send it to the idle processor
		return SC_PASS

class SC_SMS_Send(SerialCommand):
	"""Serial command to send an sms message"""
	
	def __init__(self, recipient, message):
		"""Initializes the command
		
		Parameters:
			recipient, Telephone number of the person your sending the
				message to
			
			message, The message to be sent
			
			output, output queue that should recieve the signal strength
		"""
		
		self.recipient = recipient
		self.message = message
		
		# Get the first line of the message (for echo check)
		try:
			i = message.index("\n")
		except ValueError:
			self.first_line = message
		else:
			self.first_line = message[:i]
			
		self.sent_cmgs = False
		self.sent_msg = False
		self.got_cmgs = False
	
	def GetCommand(self):
		print "Get Command..."
		
		# Check to see if the message command has been sent
		if not self.sent_cmgs:
			# Send the initial command
			self.sent_cmgs = True
			return "AT+CMGS=\"{0}\"".format(self.recipient)
			
		elif not self.sent_msg:
			# Send the message + character 26
			self.sent_msg = True
			return self.message + chr(26)
			
		# No command return a blank
		return ""
		
	def ProcessResponse(self, response, output):
		# Check for echos
		if response.startswith("AT+CMGS") or response.startswith(self.first_line):
			return SC_EXPECTED
	
		if not self.sent_msg:
			# Check for ">"
			if response == ">":
				# Ready to send the message
				return SC_EXPECTED|SC_NEXT
			elif response == "ERROR":
				# Error...
				output.put(JBMMessage(JBM_MSG_SER_ERROR, "Failed to send SMS"))
				return SC_EXPECTED|SC_COMPLETE|SC_ERROR
		elif not self.got_cmgs:	
			# Check for command return
			# What value is tagged on after CMGS?
			if response.startswith("+CMGS"):
				self.got_cmgs = True
				return SC_EXPECTED
		else:
			# Check for OK or ERROR to complete the message send
			if response == "OK":
				# Send SMS sent message
				return SC_EXPECTED|SC_COMPLETE
			elif response == "ERROR":
				# Report the error..
				return SC_EXPECTED|SC_COMPLETE|SC_ERROR
			
		
		# Not expected, send it to the idle processor
		return SC_PASS

class SC_SMS_Read(SerialCommand):
	"""Serial command to read an sms message"""
	
	def __init__(self, index):
		"""Initializes the command
		
		Parameters:
			index, the index of the message to read
		"""
		self.index = index
		
		self.sent_cmgr = False
		self.got_cmgr = False
		
		self.msg = ""
		self.flags = 0
		self.number = ""
		
		today = datetime.date.today()
		self.datestamp = today.strftime("%y/%m/%d,%H:%M:%S")
		
	def GetCommand(self):
		
		# Check to see if the message command has been sent
		if not self.sent_cmgr:
			# Send the initial command
			self.sent_cmgr = True
			return "AT+CMGR={0}".format(self.index)
			
		# No command return a blank
		return ""
		
	def ProcessResponse(self, response, output):
		# Check for echos
		if response.startswith("AT+CMGR"):
			return SC_EXPECTED
		
		if response.startswith("+CMGR:"):
			# Paras line for status, size, date and time
			temp = response[7:].split(",")
			
			# Process status
			temp[0] = temp[0].strip("\"")
			if temp[0].startswith("REC"):
				self.flags |= SMS_RECIEVED
			if temp[0].endswith(" READ"):
				self.flags |= SMS_READ
			
			# Process the number
			self.number = temp[1].strip("\"")
			
			# Process date stamp
			self.datestamp = "{0},{1}".format(temp[3], temp[4])
			self.datestamp = self.datestamp.strip("\"")
			
			# Remember the the +CMGR was recieved
			self.got_cmgr = True
			return SC_EXPECTED
		
		# Check for an error
		if response == "ERROR":
			# Report the error
			output.put(JBMMessage(JBM_MSG_SER_ERROR,
					"Error retrieving SMS"))
			return SC_EXPECTED|SC_COMPLETE|SC_ERROR
		
		if self.got_cmgr:
			# Check for OK
			if response == "OK":
				# Clean up the msg
				self.msg = self.msg.strip()
					#def __init__(self, flags, number, message, datestamp):
				# Form the sms
				output.put(JBMMessage(JBM_MSG_SMS,SMS(self.index, self.flags,
						self.number, self.msg, self.datestamp)))
						
				return SC_EXPECTED|SC_COMPLETE
			else:
				# Other data should be the message
				self.msg += "\r\n" + response
				return SC_EXPECTED
		
		return SC_PASS

class SC_SMS_List(SerialCommand):
	"""Serial command to list sms message"""
	
	def __init__(self, type, callback):
		"""Initializes the command
		
		Parameters:
			type, should be one of the following:
				ALL, all messages
				REC UNREAD, recieved messages that are unread
				REC READ, recieved messages that are read
				STO UNREAD, stored messages that are unread
				STO READ, stored messages that are read
				
			callback, function that will process the responce
		"""

		self.type = type
		self.callback = callback
		
		self.sent_cmgl = False
		self.got_cmgl = False
		
		
		# Create a date stamp for today
		today = datetime.date.today()
		self.datestamp_today = today.strftime("%y/%m/%d,%H:%M:%S")
	
		# Create an empty list to hold messages
		self.msgs = []
		
		# Initialize temporary message variables
		self.index = -1
		self.msg = ""
		self.flags = 0
		self.number = ""
		self.datestamp = self.datestamp_today
		
	def GetCommand(self):
		
		# Check to see if the message command has been sent
		if not self.sent_cmgl:
			# Send the initial command
			self.sent_cmgr = True
			return "AT+CMGL=\"{0}\"".format(self.type)
			
		# No command return a blank
		return ""
		
	def ProcessResponse(self, response, output):
		# Check for echos
		if response.startswith("AT+CMGL"):
			return SC_EXPECTED
		
		if response.startswith("+CMGL:"):
			# Check to see if a message has already been parased
			if not self.number == "":
				# Append the message to the SMS list
				self.msgs.append(SMS(self.index, self.flags,
						self.number, self.msg.strip(), self.datestamp))
				
				# Reset the message
				self.index = 0
				self.msg = ""
				self.flags = 0
				self.number = ""
				self.datestamp = self.datestamp_today
				
			
			# Paras line for status, size, date and time
			temp = response[7:].split(",")
			
			# Process index
			self.index = int(temp[0])
			
			# Process status
			temp[1] = temp[1].strip("\"")
			if temp[1].startswith("REC"):
				self.flags |= SMS_RECIEVED
			if temp[1].endswith(" READ"):
				self.flags |= SMS_READ
			
			# Process the number
			self.number = temp[2].strip("\"")
			
			# Process date stamp
			self.datestamp = "{0},{1}".format(temp[4], temp[5])
			self.datestamp = self.datestamp.strip("\"")
			
			# Remember the the +CMGR was recieved
			self.got_cmgl = True
			return SC_EXPECTED
		
		elif response == "OK":
			# Check to see if there is any message to append
			if not self.index == -1:
				# End of the list
				# Append the message to the SMS list
				self.msgs.append(SMS(self.index, self.flags,
						self.number, self.msg.strip(), self.datestamp))
						
				# Report the list
				output.put(JBMMessage(JBM_MSG_SMS_LIST, self.msgs, self.callback))
				
			return SC_EXPECTED|SC_COMPLETE
			
		elif self.got_cmgl:
			# Response is part of the message
			# Other data should be the message
			self.msg += "\r\n" + response
			return SC_EXPECTED
		
		elif response == "ERROR":
			# Report the error
			output.put(JBMMessage(JBM_MSG_SER_ERROR,
					"Error retrieving SMS", self.callback))
			return SC_EXPECTED|SC_COMPLETE|SC_ERROR
		
		return SC_PASS

class SC_SMS_Delete(SerialCommand):
	"""Serial command to delete an sms message"""
	
	def __init__(self, index):
		"""Initializes the command
		
		Parameters:
			index, the index of the message to read
		"""
		self.index = index
		self.sent_cmgd = False
		
	def GetCommand(self):
		# Check to see if the message command has been sent
		if not self.sent_cmgd:
			# Send the initial command
			self.sent_cmgd = True
			return "AT+CMGD={0}".format(self.index)
			
		# No command return a blank
		return ""
		
	def ProcessResponse(self, response, output):
		# Check for echos
		if response.startswith("AT+CMGD"):
			return SC_EXPECTED
		
		# Check for default responces
		return SerialCommand.ProcessResponse(self, response, output)

class SC_Info(SerialCommand):
	"""Serial command to return information about the modem"""
	
	def __init__(self, callback):
		"""Initializes the command
		
		Parameters:
			callback, function that will process returned information
		"""
		
		# Initiate command
		self.commands = ["AT+GMI", "AT+GMM",
				"AT+GSN", "AT+GMR", "AT+CIMI"]

		self.callback = callback
		self.data = {}
		self.expect_value = False
		
	def GetCommand(self):
		# Check to see if there are more commands to send
		if len(self.commands) == 0:
			return ""
		
		# Get the next command
		self.command = self.commands.pop()
		
		# Return the command
		return self.command
		
	def ProcessResponse(self, response, output):
		# Check for echos
		if response.startswith(self.command):
			self.expect_value = True
			return SC_EXPECTED
		
		# Check for errors 
		if response == "ERROR":
			# Error reported, abandon command
			return SC_EXPECTED|SC_ERROR|SC_COMPLETE
		elif response == "OK":
			# Current command is done, check to see if there is more to
			#	be sent
			if len(self.commands) == 0:
				# Command is done
				# Report output
				output.put(JBMMessage(JBM_MSG_INFO, self.data, self.callback))
				# Return complete status
				return SC_EXPECTED|SC_COMPLETE
				
			# Reset expect value
			self.expect_value = False
			
			# Return section complete status
			return SC_EXPECTED|SC_NEXT
		elif self.expect_value:
			# Return value is expected
			self.data[self.command[3:]] = response
			
			# Return expected status
			return SC_EXPECTED
			
		# Responce is unexpected pass it on
		return SC_PASS


class JBMSerial(threading.Thread):
	"""Provides the serial communications for Jon's Broadband Manager"""
	
	def __init__(self, output_queue, config):
		"""Initializes class...
			
		Parameters:
			output_queue, queue where output messages are placed
			
			port, serial port to connect to
		"""
		
		# Initialize the thread
		threading.Thread.__init__(self)
		
		# Remember the output queue
		self.output = output_queue
		
		# Create a command queue
		self.command_queue = Queue.Queue()

		# Create an event to flag when the thread should close
		self.keep_going = threading.Event()
		self.keep_going.set()
		
		# Load the serial port configuration
		self.baudrate = config.get("baud","serial",9600)
		self.timeout = config.get("timeout","serial",0.5)
		self.port = config.get("port","serial","/dev/ttyUSB1")
		
		# Initiate other variables
		self.tty = None
		self.service_menu = False 	# Variable indicating if the service
									#	menu is open (monitors for extra errors)
		
			
	def get_line(self):
		"""Returns the next line of data from the buffer"""

		# Initiate variables
		in_quotes = False
		eol = False
		
		# Clear any leading spaces (should this really be done??)
		self.buffer = self.buffer.lstrip()
		
		# Scan through the buffer
		for i in range(0, len(self.buffer)):
			# Check for a quote
			if self.buffer[i] == "\"":
				in_quotes = not in_quotes
			elif self.buffer[i] == "\n" and not in_quotes:
				eol = True
				break
			
		# Check to see if an end of line occured	
		if eol:
			# End of line found get the line, and remove it from the
			#  buffer
			line = self.buffer[:i]
			self.buffer = self.buffer[i+1:]
			
			# Remove white spaces from the begining and end of the line
			line = line.strip()
			
			#print "Serial: " + line
			
			# Return the line
			return line
		elif self.buffer == "> ":
			# Special case for sending SMSs no new line is sent
			self.buffer = ""
			
			return ">"
			
		# Return a blank
		return ""
			
	def __open_tty(self):
		"""Attempts to open the tty port, returns true on success"""
		
		# Create the tty object if required
		if self.tty == None:
			self.tty = serial.Serial()
		
		# Check to see if the port is open
		if self.tty.isOpen():
			return True
		
		# Set port parameters
		self.tty.baudrate = self.baudrate
		self.tty.bytesize = serial.EIGHTBITS
		self.tty.parity = serial.PARITY_NONE
		self.tty.stopbits = serial.STOPBITS_ONE
		self.tty.timeout = self.timeout
		self.tty.xonxoff = 0
		self.tty.rtscts = 0
		self.tty.interCharTimeout = None
		self.tty.port = self.port
		
		# Try to open the port
		try:
			self.tty.open()
		except serial.SerialException:
			return False
		
		# Check to see if the port opened
		if self.tty.isOpen():
			return True
		
		# Default to returning false
		return False
		
	def run(self):
		"""Main thread function"""
		
		# Initiate variables
		self.buffer = ""
		retval = SC_COMPLETE
		command = None
		command_timeout = time.time() + 100000
		
		# Try to open the port
		if not self.__open_tty():
			return
		
		# For easy...
		queue = self.command_queue
		tty = self.tty
		
		# Tell the main thread the serial port is open
		self.output.put(JBMMessage(JBM_MSG_SER_OPEN))

		# Continue until the event is cleared or the tty is closed
		while self.keep_going.is_set() and tty.isOpen():	
			# Check to see if a new command needs to be started
			if ((retval & SC_COMPLETE) > 0) or (command == None):
				# Check to see if there is a command to send
				try:
					command = queue.get(False)
				except Queue.Empty:
					command = None
					retval = SC_PASS
				else:
					retval = SC_EXPECTED|SC_NEXT
					
					# Process any data currently in the buffer to avoid
					#	any mixed up results
					line = self.get_line()
					while not line == "":
						self.process_idle(line)
			
			# Check to see if the next portion of the command should
			#	be sent (or first portion for new commands)
			if (retval & SC_NEXT) > 0:	
				# Get the next output string
				output_str = command.GetCommand()
				
				# Check for a bank string
				if not output_str == "":
					# Report what was sent
					print "Sending: {0}".format(output_str)
					
					# Try to  send the string
					try:
						tty.write(str(output_str) + "\r\n")
					except OSError:
						# OS Error means the port is probably closed
						#	kill the thread
						break
					else:
						# Calculate initial time out
						command_timeout = time.time() + SC_TIMEOUT
						

			# Read new data
			self.buffer += tty.read()
			
			# Get the next line
			line = self.get_line()
			
			# Check to see if there was a new line
			if not line == "":
				# Let the command try to process the line
				if not command == None:
					retval = command.ProcessResponse(line, self.output)
				
				# Check to see if the value should be passed on
				if retval == SC_PASS:
					if self.process_idle(line):
						# Reset the timeout
						command_timeout = time.time() + SC_TIMEOUT
				else:
					# Recalculate a time out
					if (retval & SC_LONG) > 0:
						command_timeout = time.time() + SC_TIMEOUT_LONG
					else:
						command_timeout = time.time() + SC_TIMEOUT
					
			else:
				# Check to see if there is any data in the buffer
				if self.buffer == "":
					# Sleep for a bit
					if command == None:
						time.sleep(1.0)
					else:
						time.sleep(0.10)
					
				# Default the return value to pass
				retval = SC_PASS
			
			# Check to see if the service menu is open
			if ((retval & SC_SERVICE) > 0):
				print "Serial: Service Menu Opened"
				self.service_menu = True
			
			# Check to see if the command is done
			if ((retval & SC_COMPLETE) > 0):
				# Indicate the command is done to the queue
				queue.task_done()
				
				# Clear the command
				command = None
			
			# Check for timeouts
			if not command == None and time.time() > command_timeout:
				# Report error
				print "Serial: Command Timeout"
				
				try:
					self.output.put(JBMMessage(JBM_MSG_SER_TIMEOUT,type(command).__name__,command.callback))
				except AttributeError:
					self.output.put(JBMMessage(JBM_MSG_SER_TIMEOUT,type(command).__name__))
					
				# Indicate the command is done to the queue
				queue.task_done()
				
				# Clear the command
				command = None
				
		# Close the serial port
		tty.close()
		
		# Delete the tty object
		del tty

		# Tell the main thread that the serial port is closed
		self.output.put(JBMMessage(JBM_MSG_SER_CLOSED))
			
	def process_idle(self, line):
		"""Processes idle communications from the modem
		
		Return true if timeout should be reset, good for long commands
		
		Possible Idle Message		
		+ZONDR:
		"""
		
		if line == "+ZUSIMR:2":
			# Unknown error code, clear it by send CPMS=?
			self.command_queue.put(SC_CPMS())

		elif line.startswith("+CMTI: \"ME\""):
			# New SMS!
			
			# Sleep the thread for a bit (in case an extra error is sent)
			print "Serial: Got CMTI"
			time.sleep(2)
			
			# Report the new SMS
			self.output.put(JBMMessage(JBM_MSG_SMS_REC,int(line[12:])))
			
			return True
		elif line.startswith("+ZPASR"):
			# Network change
			temp = line.split("\"")
			if not temp[1] == "No Service" and not temp[1] == "Limited Service":
				self.output.put(JBMMessage(JBM_MSG_NETWORK, temp[1]))
			print "Serial: Connection " + temp[1]
			
		elif line == "OK":
			# Someone forgot to process an OK...
			print "Serial: Extra OK Found"
			
		elif line == "ERROR":
			# Check to see if the service menu is open
			if self.service_menu:
				print "Serial: Service Menu Timeout"
				# Report the service menu has closed
				self.output.put(JBMMessage(JBM_MSG_CUSD, ""))
				return
				
			# Hrm...
			print "Serial: Unhandled Error Found"
			
		else:
			print "Serial: Unknown Response: " + line
		
		# Return false, no delay required
		return False
		
	def send(self, command):
		"""Adds the specified command to the command queue"""
		
		self.command_queue.put(command)
		
	def close(self):
		"""Kills the serial port"""
		
		# Unset the keep going flag (should kill the thread)
		self.keep_going.clear()
		
		# Join the thread...
		self.join(2.0)

# *************** Main *************
